//                                                                                
//    Copyright (c) 2004/2005 Thomas Fuchs / The Rockin'-B, www.rockin-b.de       
//                                                                                
//    Permission is granted to anyone to use this software for any purpose        
//    and to redistribute it freely, subject to the following restrictions:       
//                                                                                
//    1.  The origin of this software must not be misrepresented. You must not    
//        claim that you wrote the original software. If you use this software    
//        in a product, an acknowledgment in the product documentation would be   
//        appreciated but is not required.                                        
//                                                                                
//    2.  ANY COMMERCIAL USE IS PROHIBITED. This includes that you must not use   
//        this software in the production of a commercial product. Only an explicit
//        permission by the author can allow you to do that.                      
//                                                                                
//    3.  Any modification applied to the software must be marked as such clearly.
//                                                                                
//    4.  This license may not be removed or altered from any distribution.       
//                                                                                
//                                                                                
//    			    NO WARRANTY                                                 
//                                                                                
//    THERE IS NO WARRANTY FOR THE PROGRAM.                                       
//    THE PROGRAM IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,               
//    EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,                 
//    THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE.                 
//    THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.
//    SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY    
//    SERVICING, REPAIR OR CORRECTION.                                            
//                                                                                
//    IN NO EVENT WILL THE COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES,         
//    INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
//    OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED   
//    TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY    
//    YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER  
//    PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE       
//    POSSIBILITY OF SUCH DAMAGES.                                                
//                                                                                

#include <stdlib.h>
#include <SGL.H>
#include <SGM_def.h>
#include <SGM_BZip2.h>


void set_date(MemoryFile *mFile)
{
    SmpcDateTime *time;
	time = &(Smpc_Status->rtc);
	mFile->year  = (Uint16)slDec2Hex((Uint32)time->year);
	mFile->month = (Uint8)(time->month & 0x0f);
	mFile->weekday = (Uint8)(time->month >> 4);
	mFile->day = (Uint8)(slDec2Hex((Uint32)time->date));
	mFile->hour = (Uint8)(slDec2Hex((Uint32)time->hour));
	mFile->minute = (Uint8)(slDec2Hex((Uint32)time->minute));
	mFile->language = BUP_ENGLISH;
}
/****************************************************************************************************/
/* BZip2 compression ********************************************************************************/
/****************************************************************************************************/

#include <bzlib.h>
#define COMPRESS_BUFFER_UNIT    1024

void bz_internal_error ( int errcode ) {}

int blockSize100k = 1;
int small = 1;
    // blockSize100k    compression:    maxBlock    maxTotal    decompression:  maxBlock    maxTotal
    // 1                                400.136     1.118.011                   200.000     250.000
    // 2                                800.136     1.918.011                   400.000     500.000
    // 3                                1.200.136   2.718.011                   600.000     750.000
    // 4                                1.600.136   3.518.011                   800.000     1.000.000
    // !!! ---> we cannot neccessarily assume that
    //          BGET can place the biggest chunk into
    //          the correct (one and only fitting) biggest area
    //
    // blocksize   cart type 
    // 1           not matter
    // 2           (8M no, as it's split in 2 banks) + 32MB
    // 3           32M
    // 4           32M

// NEED custom memory allocator
#ifndef USE_BGET
#include <SEGA_MEM.H>

void my_bzallocInit()
{
    MEM_Init(LOW_MEM_START, LOW_MEM_SIZE);
}

void *my_bzalloc(void *opaque, int items, int size)
{
    void *v;
    // put everything into low work RAM
    v = MEM_Malloc(items * size);
    // if low is full, take high work RAM
    if(v == NULL) {
        v = malloc(items * size);
    }
    
    return v;
}

void my_bzfree(void *opaque, void *addr)
{
    if(addr != NULL) {
        if(((unsigned int)addr) <= ((unsigned int)(LOW_MEM_START + LOW_MEM_SIZE))) {
            MEM_Free(addr);
        } else {
            free(addr);
        }
    }
}
#else
void *my_bzalloc(void *opaque, int items, int size)
{
   void* v = (void *)malloc ( items * size );
   return v;
}

void my_bzfree(void *opaque, void *addr)
{
   if (addr != NULL) free ( addr );
}
#endif

int compressAllInput(bz_stream *strm, void **buffer, void *data, Uint32 size)
{
    void *ptr;
    int ret;
    
    strm->avail_in = size;
    strm->next_in = data;
    
    do {
        // increase output buffer?
        if(strm->avail_out == 0) {
            if(strm->next_out == NULL)
                ptr = (void *)malloc(COMPRESS_BUFFER_UNIT);
            else ptr = (void *)realloc(*buffer, strm->total_out_lo32 + COMPRESS_BUFFER_UNIT);
            if(ptr == NULL) {
                    // free compressed output
                if(strm->next_out != NULL) //if(strm->total_out_lo32 > 0)
                    free(*buffer);
                    // free bzip2 work
                BZ2_bzCompressEnd(strm);
                return -2;
            }
            *buffer = ptr;
            strm->avail_out = COMPRESS_BUFFER_UNIT;
            strm->next_out = ptr + strm->total_out_lo32;
        }
        ret = BZ2_bzCompress(strm, BZ_RUN);
        if(ret != BZ_RUN_OK) {
                // free compressed output
            //if(strm->total_out_lo32 > 0)
                free(*buffer);
                // free bzip2 work
            BZ2_bzCompressEnd(strm);
            return -3;
        }
    } while(strm->avail_in > 0);
    
    return 0;
}

char getHexDigit(Uint8 nibble)
{
    nibble &= 0xF;
    if(nibble <= 9)
        return '0' + nibble;
    else return 'a' + nibble - 10;
}

int compressDeviceToFile(struct STORAGE_DEVICE *sDevice, MemoryFile *mFile)
{
    Uint32 i, n;
    int ret;
    bz_stream strm;
    MemoryFile tmpFile;
    // check args
    if((sDevice == NULL) || (mFile == NULL))
        return -1;
    // as this function deletes all file data on ERROR
    // this must be set to detect if data is present outside
    // by checking file size
    mFile->size = 0;
    // init bz2
        // custom memory allocator
#ifndef USE_BGET
    my_bzallocInit();
#endif
    strm.bzalloc = my_bzalloc;
    strm.bzfree = my_bzfree;
    strm.opaque = NULL;
        // IMPORTAND for compressAllInput to alloc correctly
    strm.avail_out = 0;
    strm.next_out = NULL;
        // blockSize100k: 1-9, 9 = best compression & most memory usage
        // verbosity level 0-4, have no stdio, so 0
        // workFactor: use default 30
    ret = BZ2_bzCompressInit(&strm, blockSize100k, 0, 30);
    if(ret != BZ_OK)
        return -3;
    // compress the drive
        // number of files
    n = sDevice->getFileNum(sDevice);
    ret = compressAllInput(&strm, &(mFile->data), (void *)&n, sizeof(Uint32));
    if(ret != 0)
        return ret;
        // each file
    for(i = 0; i < n; i++) {
            // load
        ret = sDevice->loadFile(sDevice, i, &tmpFile);
        if(ret != 0) {
                // free temporary file
            if(tmpFile.size > 0)
                free(tmpFile.data);
                // free compressed output
            //if(strm.total_out_lo32 > 0)
                free(mFile->data);
                // free bzip2 work            
            BZ2_bzCompressEnd(&strm);
            return ret;
        }
            // compress MemoryFile structure
        ret = compressAllInput(&strm, &(mFile->data), (void *)&tmpFile, sizeof(MemoryFile));
        if(ret != 0) {
                // free temporary file
            if(tmpFile.size > 0)
                free(tmpFile.data);
            return ret;
        }    
        // handle empty files
        // BZip2: "You should always ensure that at least one byte of output space is available at each call"
        if(tmpFile.size > 0) {
                // compress data
            ret = compressAllInput(&strm, &(mFile->data), (void *)tmpFile.data, tmpFile.size);
            if(ret != 0) {
                    // free temporary file
                if(tmpFile.size > 0)
                    free(tmpFile.data);
                return ret;
            }    
                // free temporary file
            free(tmpFile.data);
        }    
    }
    // finish
    do {
        void *ptr;
        // increase output buffer?
        if(strm.avail_out == 0) {
            if(strm.next_out == NULL)
                ptr = (void *)malloc(COMPRESS_BUFFER_UNIT);
            else ptr = (void *)realloc(mFile->data, strm.total_out_lo32 + COMPRESS_BUFFER_UNIT);
            if(ptr == NULL) {
                    // free compressed output
                //if(strm.next_out != NULL) //if(strm.total_out_lo32 > 0)
                    free(mFile->data);
                    // free bzip2 work
                BZ2_bzCompressEnd(&strm);
                return -2;
            }
            mFile->data = ptr;
            strm.avail_out = COMPRESS_BUFFER_UNIT;
            strm.next_out = ptr + strm.total_out_lo32;
        }
        ret = BZ2_bzCompress(&strm, BZ_FINISH);
        if((ret == BZ_SEQUENCE_ERROR) || (ret == BZ_PARAM_ERROR)) {
                // free compressed output
            //if(strm.total_out_lo32 > 0)
                free(mFile->data);
                // free bzip2 work            
            BZ2_bzCompressEnd(&strm);
            return -3;
        }
    } while(ret != BZ_STREAM_END);
    // write file attributes
    set_date(mFile);
	mFile->size = strm.total_out_lo32;
	// preserve CUSTOM drive name
	if(sDevice->name[11] == '\0')
        memcpy(mFile->name, sDevice->name, 12);
    else {
           // name is time dependent
    	   // a different filename depending on
    	   // time and size
        n = ((Uint32)(mFile->year-1980))*360*24*60 + ((Uint32)(mFile->hour))*60 + ((Uint32)mFile->minute) + mFile->size;
        mFile->name[0] = 'S';
        mFile->name[1] = 'G';
        mFile->name[2] = 'M';
        for(i = 0; i < 8; i++)
            mFile->name[3+i] = getHexDigit(n >> (32-4*(i+1)));
        mFile->name[11] = '\0';
    }
//    snprintf(mFile->name, 11, "SGM_%7x", n);
    char comment[] = "VirtualDrv";
    memcpy(mFile->comment, comment, 11);
//    snprintf(mFile->comment, 10, "VirtualDrv");
    ret = BZ2_bzCompressEnd(&strm);
    if(ret != BZ_OK)
        return -3;
    
    return 0;
}

// assumes that decompress does fill the whole output buffer
// except if uncompressed format of input file differs
int decompressFileToDevice(struct STORAGE_DEVICE *sDevice, MemoryFile *mFile)
{
    Uint32 i, n;
    int ret;
    bz_stream strm;
    MemoryFile tmpFile;
    // check args
    if((sDevice == NULL) || (mFile == NULL))
        return -100;
        // small file type test
    if((mFile->name[0] != 'S') || (mFile->name[1] != 'G') || (mFile->name[2] != 'M'))
        return -100;
    // init bz2
        // custom memory allocator
#ifndef USE_BGET
    my_bzallocInit();
#endif
    strm.bzalloc = my_bzalloc;
    strm.bzfree = my_bzfree;
    strm.opaque = NULL;
        // verbosity level 0-4, have no stdio, so 0
        // small: non-zero to drop max memory requirements to 2300k
    ret = BZ2_bzDecompressInit(&strm, 0, small);
    if(ret != BZ_OK)
        return ret;
    // decompress
    strm.avail_in = mFile->size;
    strm.next_in = mFile->data;
        // number of files
    strm.next_out = (void *)&n;
    strm.avail_out = sizeof(Uint32);
    ret = BZ2_bzDecompress(&strm);
    if((ret != BZ_OK) && (ret != BZ_STREAM_END)) {
        BZ2_bzDecompressEnd(&strm);
        return ret;
    }
        // all files
    for(i = 0; i < n; i++) {
        // decompress MemoryFile structure
        strm.next_out = (void *)&tmpFile;
        strm.avail_out = sizeof(MemoryFile);
        ret = BZ2_bzDecompress(&strm);
        if(ret != BZ_OK) {
            BZ2_bzDecompressEnd(&strm);
            return ret;
        }
        // handle empty files
        // BZip2: "You should always ensure that at least one byte of output space is available at each call"
        if(tmpFile.size > 0) {
            // decompress data
                // allocate temporary data buffer
            tmpFile.data = (void *)malloc(tmpFile.size);
            if(tmpFile.data == NULL) {
                BZ2_bzDecompressEnd(&strm);
                return -200;
            }
                // decompress
            strm.next_out = tmpFile.data;
            strm.avail_out = tmpFile.size;
            ret = BZ2_bzDecompress(&strm);
            // stream can end here for the last file
            if((ret != BZ_OK) && !((ret == BZ_STREAM_END) && (i == (n-1)))) {
                if(tmpFile.size > 0)
                    free(tmpFile.data);
                BZ2_bzDecompressEnd(&strm);
                return ret;
            }
        }    
        // save file
        ret = sDevice->saveFile(sDevice, &tmpFile);
        if(ret != 0) {
            if(tmpFile.size > 0)
                free(tmpFile.data);
            BZ2_bzDecompressEnd(&strm);
            return -300;
        }
        // free temporary memory
        if(tmpFile.size > 0)
            free(tmpFile.data);
    }
    // assume that last decompress returned BZ_STREAM_END
    ret = BZ2_bzDecompressEnd(&strm);
    if(ret != BZ_OK) 
        return ret;    
        
    return 0;
}

