//                                                                                
//    Copyright (c) 2004/2005 Thomas Fuchs / The Rockin'-B, www.rockin-b.de       
//                                                                                
//    Permission is granted to anyone to use this software for any purpose        
//    and to redistribute it freely, subject to the following restrictions:       
//                                                                                
//    1.  The origin of this software must not be misrepresented. You must not    
//        claim that you wrote the original software. If you use this software    
//        in a product, an acknowledgment in the product documentation would be   
//        appreciated but is not required.                                        
//                                                                                
//    2.  ANY COMMERCIAL USE IS PROHIBITED. This includes that you must not use   
//        this software in the production of a commercial product. Only an explicit
//        permission by the author can allow you to do that.                      
//                                                                                
//    3.  Any modification applied to the software must be marked as such clearly.
//                                                                                
//    4.  This license may not be removed or altered from any distribution.       
//                                                                                
//                                                                                
//    			    NO WARRANTY                                                 
//                                                                                
//    THERE IS NO WARRANTY FOR THE PROGRAM.                                       
//    THE PROGRAM IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,               
//    EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,                 
//    THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE.                 
//    THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.
//    SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY    
//    SERVICING, REPAIR OR CORRECTION.                                            
//                                                                                
//    IN NO EVENT WILL THE COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES,         
//    INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
//    OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED   
//    TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY    
//    YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER  
//    PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE       
//    POSSIBILITY OF SUCH DAMAGES.                                                
//                                                                                

#include <SGM_def.h>
#include <SGM_IO_pad.h>
#include <SGM_popup.h>


extern void checkExitHere(Uint16 data);
extern void clearScreen();


static Uint32 table[8] = {
    PER_DGT_TA,
    PER_DGT_TB,
    PER_DGT_TC,
    PER_DGT_TX,
    PER_DGT_TY,
    PER_DGT_TZ,
    PER_DGT_TL,
    PER_DGT_TR,
};

static char *buttonNames[16] = {
    "    A    ",
    "    B    ",
    "    C    ",
    "    X    ",
    "    Y    ",
    "    Z    ",
    "    L    ",
    "    R    ",
    "START + A",
    "START + B",
    "START + C",
    "START + X",
    "START + Y",
    "START + Z",
    "START + L",
    "START + R",
};


Uint8 readNibble(MemoryFile *mFile, Uint8 *data, Uint32 n)
{
    Uint8 byte;
    
    if(n < (sizeof(MemoryFile) << 1)) {
        // access MemoryFile
        data = (Uint8 *)mFile;
    } else {
        // access data
        n -= (sizeof(MemoryFile) << 1);
    }
    byte = *(data + (n >> 1));
    
    if(n & 1)
        byte >>= 4;     // upper nibble
    else byte &= 0xF;   // lower nibble
    
    return byte;
}

void writeNibble(MemoryFile *mFile, Uint8 *data, Uint32 n, Uint8 nibble)
{
    Uint8 byte;

    if(n < (sizeof(MemoryFile) << 1)) {
        // access MemoryFile
        data = (Uint8 *)mFile;
    } else {
        // access data
        n -= (sizeof(MemoryFile) << 1);
    }
    byte = *(data + (n >> 1));
    
    if(n & 1)
        byte = ((nibble << 4) & 0xF0) | (byte & 0x0F);  // upper nibble
    else byte = (byte & 0xF0) | (nibble & 0x0F);        // lower nibble
    
    *(data + (n >> 1)) = byte;

    return;
}


int SGM_IO_pad_append(Uint8 **data, Uint32 *bytes)
{
    Uint8 *ptr;
    // check args
    if((data == NULL) || (bytes == NULL))
        return -1;
    // alloc
    if((*data == NULL) || (*bytes == 0))
        ptr = (Uint8 *)malloc(sizeof(Uint8));
    else ptr = (Uint8 *)realloc(*data, *bytes + sizeof(Uint8));
    if(ptr == NULL)
        return -2;
    // set data
    *(ptr + *bytes) = 0;
    *data = ptr;
    *bytes += 1;

    return 0;
}

int SGM_IO_pad_remove(Uint8 **data, Uint32 *bytes)
{
    Uint8 *ptr;
    // check args
    if((data == NULL) || (bytes == NULL))
        return -1;
    // remove
    if((*bytes == 1)) {
        free(*data);
        *data = NULL;
    } else if(*bytes > 1) {
        ptr = (Uint8 *)realloc(*data, *bytes - sizeof(Uint8));
        if(ptr == NULL)
            return -2;
        *data = ptr;
    } else return 0;
    *bytes -= 1;

    return 0;
}

char *BUTTONEDIT[] = {
    "                                            ",
    "           Button Command Editor            ",
    "                                            ",
    "  length:        |  index:|commands:|cursor:",
    "                 |--------|---------|-------",
    "                 |        |         |       ",
    "  exit:          |        |         |       ",
    "   START + UP    |        |         |       ",
    "   START + DOWN  |        |         |       ",
    "                 |        |         |       ",
    "  cursor:        |        |         |       ",
    "   UP/DOWN       |        |         |       ",
    "                 |        |         |       ",
    "  page:          |        |         |       ",
    "   LEFT/RIGHT    |        |         |       ",
    "                 |        |         |       ",
    "  page checksum: |        |         |       ",
    "                 |        |         |       ",
    "                 |        |         |       ",
    "  set command:   |        |         |       ",
    "   (START + )    |        |         |       ",
    "    ABCXYZLR     |        |         |       ",
    "                 |        |         |       ",
    "  append command:|        |         |       ",
    "   START + RIGHT |        |         |       ",
    "                 |        |         |       ",
    "  remove command:|        |         |       ",
    "   START + LEFT  |        |         |       ",
    "                                            ",
    "              www.rockin-b.de               ",
};

char *BUTTONVIEW[] = {
    "                                            ",
    "           Button Command Viewer            ",
    "                                            ",
    "  length:        |  index:|commands:|cursor:",
    "                 |--------|---------|-------",
    "                 |        |         |       ",
    "  exit:          |        |         |       ",
    "   START + UP    |        |         |       ",
    "   START + DOWN  |        |         |       ",
    "                 |        |         |       ",
    "  cursor:        |        |         |       ",
    "   UP/DOWN       |        |         |       ",
    "                 |        |         |       ",
    "  page:          |        |         |       ",
    "   LEFT/RIGHT    |        |         |       ",
    "                 |        |         |       ",
    "  page checksum: |        |         |       ",
    "                 |        |         |       ",
    "                 |        |         |       ",
    "                 |        |         |       ",
    "                 |        |         |       ",
    "                 |        |         |       ",
    "                 |        |         |       ",
    "                 |        |         |       ",
    "                 |        |         |       ",
    "                 |        |         |       ",
    "                 |        |         |       ",
    "                 |        |         |       ",
    "                                            ",
    "              www.rockin-b.de               ",
};

void SGM_IO_pad_draw(MemoryFile *mFile, Uint8 *data, Uint32 bytes, Uint32 startNibble, Uint32 cursor, Uint8 edit)
{
    int i;
    Uint32 nibble = startNibble;
    Uint32 checksum = 0;
    Uint8 read;
    
    // background
    if(edit) {
        for(i = 0; i < (sizeof(BUTTONEDIT)/sizeof(char *)); i++)
            slPrint(*(BUTTONEDIT + i), slLocate(0, i));
    } else {
        for(i = 0; i < (sizeof(BUTTONVIEW)/sizeof(char *)); i++)
            slPrint(*(BUTTONVIEW + i), slLocate(0, i));
    }    
    // button commands
    for(i = 5; (i < 28) && (nibble < ((sizeof(MemoryFile) + bytes) << 1)); i++, nibble++) {
        // nibble number
        slPrintHex(nibble, slLocate(18, i));
        // button name
        read = readNibble(mFile, data, nibble);
        slPrint(buttonNames[read], slLocate(27, i));
        checksum += read;
    }
    // print cursor
    if((startNibble <= cursor) && (cursor <= (startNibble+23)))
        slPrint("<---", slLocate(37, 5 + cursor - startNibble));
    // values
    slPrintFX((sizeof(MemoryFile) + bytes) << 17, slLocate(2, 4));
    slPrintFX((cursor / 23) << 16, slLocate(8, 13));
    slPrintHex(checksum, slLocate(8, 17));
}

// 
void SGM_IO_pad(MemoryFile *mFile, Uint8 edit)
{
    Sint32 nibble = 0;
    Uint8 redraw;
    Uint8 *data;
    Uint32 bytes;

    if(mFile == NULL)
        return;
        
    data = mFile->data;
    bytes = mFile->size;
    
    SGM_IO_pad_draw(mFile, data, bytes, (nibble / 23) * 23, nibble, edit);
    waitRelease(PER_DGT_ST | PER_DGT_KU | PER_DGT_KD | PER_DGT_TA | PER_DGT_TB | PER_DGT_TC | PER_DGT_TX | PER_DGT_TY | PER_DGT_TZ | PER_DGT_TL | PER_DGT_TR);
    
    while(1) {
        Uint16 pad, push;
        getfirstPadInput(&pad, &push, NULL);
        redraw = 1;
        // check free nibble
        if(nibble >= ((sizeof(MemoryFile) + bytes) << 1)) {
            SGM_IO_pad_append(&data, &bytes);
            nibble = ((sizeof(MemoryFile) + bytes) << 1) - 1 - 1;
        }
        // inputs    
            // exit ?
        if(((pad & PER_DGT_ST) == 0) && (((push & PER_DGT_KU) == 0) || ((push & PER_DGT_KD) == 0))) {
            if(mFile->size != bytes)
                SGM_confirm(0xFFFF, CONFIRM_ERROR, "length corrected");
            return;
        }

            // view functions
        if((pad & PER_DGT_ST) != 0) {
                // only if no other input occured
                // change selected nibble
            if( ((push & PER_DGT_KU) == 0) || ( ((pad & PER_DGT_KU) == 0) && ( ((pad & PER_DGT_TL) == 0) || ((pad & PER_DGT_TR) == 0) ) ) ) {
                if(nibble > 0)
                    nibble--;
            } else if( ((push & PER_DGT_KD) == 0) || ( ((pad & PER_DGT_KD) == 0) && ( ((pad & PER_DGT_TL) == 0) || ((pad & PER_DGT_TR) == 0) ) ) ) {
                if(nibble < (((sizeof(MemoryFile) + bytes) << 1) - 1) )
                    nibble++;
            } else if( ((push & PER_DGT_KL) == 0) || ( ((pad & PER_DGT_KL) == 0) && ( ((pad & PER_DGT_TL) == 0) || ((pad & PER_DGT_TR) == 0) ) ) ) {   // fast page wise
                nibble -= 23;
                if(nibble < 0)
                    nibble = 0;
            } else if( ((push & PER_DGT_KR) == 0) || ( ((pad & PER_DGT_KR) == 0) && ( ((pad & PER_DGT_TL) == 0) || ((pad & PER_DGT_TR) == 0) ) ) ) {
                nibble += 23;
                if(nibble >= ((sizeof(MemoryFile) + bytes) << 1) )
                    nibble = ((sizeof(MemoryFile) + bytes) << 1) - 1;
            } else redraw = 0;
        } else redraw = 0;
            // edit functions
        if((redraw == 0) && (edit != 0)) {
            redraw = 1;
            if((pad & PER_DGT_ST) == 0) {
                if(((push & PER_DGT_KL) == 0)) { // && (nibble >= ((*bytes << 1) - 2))) {
                    SGM_IO_pad_remove(&data, &bytes);
                    nibble = ((sizeof(MemoryFile) + bytes) << 1) - 1;
                } else if(((push & PER_DGT_KR) == 0)) { // && (nibble >= ((*bytes << 1) - 2))) {
                    SGM_IO_pad_append(&data, &bytes);
                    nibble = ((sizeof(MemoryFile) + bytes) << 1) - 1;
                } else
                    // input command (with SHIFT)
                if((push & PER_DGT_TA) == 0)
                    writeNibble(mFile, data, nibble++, 0x8);
                else if((push & PER_DGT_TB) == 0)
                    writeNibble(mFile, data, nibble++, 0x9);
                else if((push & PER_DGT_TC) == 0)
                    writeNibble(mFile, data, nibble++, 0xa);
                else if((push & PER_DGT_TX) == 0)
                    writeNibble(mFile, data, nibble++, 0xb);
                else if((push & PER_DGT_TY) == 0)
                    writeNibble(mFile, data, nibble++, 0xc);
                else if((push & PER_DGT_TZ) == 0)
                    writeNibble(mFile, data, nibble++, 0xd);
                else if((push & PER_DGT_TL) == 0)
                    writeNibble(mFile, data, nibble++, 0xe);
                else if((push & PER_DGT_TR) == 0)
                    writeNibble(mFile, data, nibble++, 0xf);
                else redraw = 0;
            } else {
                // input command (without SHIFT)
                if((push & PER_DGT_TA) == 0)
                    writeNibble(mFile, data, nibble++, 0x0);
                else if((push & PER_DGT_TB) == 0)
                    writeNibble(mFile, data, nibble++, 0x1);
                else if((push & PER_DGT_TC) == 0)
                    writeNibble(mFile, data, nibble++, 0x2);
                else if((push & PER_DGT_TX) == 0)
                    writeNibble(mFile, data, nibble++, 0x3);
                else if((push & PER_DGT_TY) == 0)
                    writeNibble(mFile, data, nibble++, 0x4);
                else if((push & PER_DGT_TZ) == 0)
                    writeNibble(mFile, data, nibble++, 0x5);
                else if((push & PER_DGT_TL) == 0)
                    writeNibble(mFile, data, nibble++, 0x6);
                else if((push & PER_DGT_TR) == 0)
                    writeNibble(mFile, data, nibble++, 0x7);
                else redraw = 0;
            }    
        };   
        // always up to date -> cannot be overwritten
        mFile->data = data;
        mFile->size = bytes;
        if(redraw) {
            // check free nibble
            if(nibble >= ((sizeof(MemoryFile) + bytes) << 1)) {
                SGM_IO_pad_append(&data, &bytes);
                nibble = ((sizeof(MemoryFile) + bytes) << 1) - 1 - 1;
            }
            SGM_IO_pad_draw(mFile, data, bytes, (nibble / 23) * 23, nibble, edit);
        }    
        slSynch();
    }
}

