//                                                                                
//    Copyright (c) 2004/2005 Thomas Fuchs / The Rockin'-B, www.rockin-b.de       
//                                                                                
//    Permission is granted to anyone to use this software for any purpose        
//    and to redistribute it freely, subject to the following restrictions:       
//                                                                                
//    1.  The origin of this software must not be misrepresented. You must not    
//        claim that you wrote the original software. If you use this software    
//        in a product, an acknowledgment in the product documentation would be   
//        appreciated but is not required.                                        
//                                                                                
//    2.  ANY COMMERCIAL USE IS PROHIBITED. This includes that you must not use   
//        this software in the production of a commercial product. Only an explicit
//        permission by the author can allow you to do that.                      
//                                                                                
//    3.  Any modification applied to the software must be marked as such clearly.
//                                                                                
//    4.  This license may not be removed or altered from any distribution.       
//                                                                                
//                                                                                
//    			    NO WARRANTY                                                 
//                                                                                
//    THERE IS NO WARRANTY FOR THE PROGRAM.                                       
//    THE PROGRAM IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,               
//    EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,                 
//    THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE.                 
//    THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.
//    SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY    
//    SERVICING, REPAIR OR CORRECTION.                                            
//                                                                                
//    IN NO EVENT WILL THE COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES,         
//    INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
//    OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED   
//    TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY    
//    YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER  
//    PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE       
//    POSSIBILITY OF SUCH DAMAGES.                                                
//                                                                                

#include <SGM_def.h>
#include <SGM_MemoryViewer.h>


#define SGM_MV_LINES                23
#define SGM_MV_BYTES_PER_LINE       10
#define SGM_MV_BYTES_PER_PAGE       (SGM_MV_LINES * SGM_MV_BYTES_PER_LINE)


// to avoid unaligned word access
Uint32 SGM_MV_readUint32(Uint8 *addr)
{
    Uint32 ret;
    if(addr == NULL)
        return 0;
        
//    ret = (*(addr + 0) & 0x000000FF) << 0;
//    ret |= (*(addr + 1) & 0x000000FF) << 8;
//    ret |= (*(addr + 2) & 0x000000FF) << 16;
//    ret |= (*(addr + 3) & 0x000000FF) << 24;
    // swap for low endian display
    ret = (*(addr + 0) & 0x000000FF) << 24;
    ret |= (*(addr + 1) & 0x000000FF) << 16;
    ret |= (*(addr + 2) & 0x000000FF) << 8;
    ret |= (*(addr + 3) & 0x000000FF) << 0;
    
    return ret;
}

// print 10 bytes per line
void SGM_MV_draw(Uint8 *addr, Uint8 *my_addr, Uint32 maxAddr)
{
    int i, j, o;
    Uint32 hex;
    char str[SGM_MV_BYTES_PER_LINE+1];
    str[SGM_MV_BYTES_PER_LINE] = '\0';
    
    slPrint("Memory Viewer", slLocate(15, 0));
    slPrint("selected address:              exit: START", slLocate(0, 2));
    slDispHex((Uint32)addr, slLocate(18, 2));

    slPrint("address:|       hex view:      |text view:", slLocate(0, 3));
    slPrint("--------|----------------------|----------", slLocate(0, 4));
//    slPrint("hex view:", slLocate(9, 3));
//    slPrint("text view:", slLocate(32, 3));
    
    for(i = 5, o = 0; i < 5+SGM_MV_LINES; i++, o+=SGM_MV_BYTES_PER_LINE) {
        // draw hex if line start is in bounds (end can go outside)
        if(((Uint32)(my_addr + o)) <= maxAddr) {
            // draw text
            for(j = 0; j < SGM_MV_BYTES_PER_LINE; j++) {
                Uint8 c = *(my_addr+o+j);
                // allow all base64 characters
//                if( ((c >= 48) && (c <= 57)) || ((c >= 65) && (c <= 90)) || ((c >= 97) && (c <= 122)) )
                if((c == '+') || (c == '/') || (c == '=') || ((c >= '0') && (c <= '9')) || ((c >= 'A') && (c <= 'Z')) || ((c >= 'a') && (c <= 'z')) )
                    str[j] = c;
                else str[j] = ' ';
            }
            slPrint(str, slLocate(32, i));
            // print rightmost only by half
            slDispHex(SGM_MV_readUint32(my_addr+o+8) >> 16, slLocate(9+8+4+1, i));
            slDispHex(SGM_MV_readUint32(my_addr+o+4), slLocate(9+8+1, i));
            slDispHex(SGM_MV_readUint32(my_addr+o), slLocate(9+1, i));
        } else {
            // out of bounds
            for(j = 0; j < SGM_MV_BYTES_PER_LINE; j++)
                str[j] = ' ';
            slPrint(str, slLocate(32, i));
            slPrint(str, slLocate(10, i));
            slPrint(str, slLocate(20, i));
        }
        // draw start addr
        slDispHex((Uint32)(my_addr + o), slLocate(0, i));
        // draw borders
        slPrint("|", slLocate(8, i));
        slPrint("|", slLocate(31, i));
    }
    slPrint("   set: A/C   www.rockin-b.de   reset: B    ", slLocate(0, 29));
}

// returns:
//  0   -   no inputs 
//  1   -   my_addr changed
// -1   -   want to return   
int SGM_MV_inputs(Uint16 push, Uint16 data, Uint8 **addr, Uint8 **my_addr)
{
    // return
    if((push & PER_DGT_ST) == 0)
        return -1;
    // set address
    else if(((push & PER_DGT_TA) == 0) || ((push & PER_DGT_TC) == 0))
        *addr = *my_addr;
    // reset address
    else if((push & PER_DGT_TB) == 0)
        *my_addr = *addr;
    // adjust address
        // page wise
    else if((data & PER_DGT_KU) == 0) {
        if((push & PER_DGT_KU) == 0)
            *my_addr -= SGM_MV_BYTES_PER_PAGE;
        else if(((data & PER_DGT_TL) == 0) && ((data & PER_DGT_TR) == 0))
            *my_addr -= 512*SGM_MV_BYTES_PER_PAGE;
        else if((data & PER_DGT_TR) == 0)
            *my_addr -= 32*SGM_MV_BYTES_PER_PAGE;
        else if((data & PER_DGT_TL) == 0)
            *my_addr -= 2*SGM_MV_BYTES_PER_PAGE;
    } else if(((push & PER_DGT_KD) == 0) || (((data & PER_DGT_KD) == 0) && ( ((data & PER_DGT_TL) == 0) || ((data & PER_DGT_TR) == 0)))) {
        if((push & PER_DGT_KD) == 0)
            *my_addr += SGM_MV_BYTES_PER_PAGE;
        else if(((data & PER_DGT_TL) == 0) && ((data & PER_DGT_TR) == 0))
            *my_addr += 512*SGM_MV_BYTES_PER_PAGE;
        else if((data & PER_DGT_TR) == 0)
            *my_addr += 32*SGM_MV_BYTES_PER_PAGE;
        else if((data & PER_DGT_TL) == 0)
            *my_addr += 2*SGM_MV_BYTES_PER_PAGE;
        // byte wise
    } else if((data & PER_DGT_KL) == 0)
        *my_addr -= 1;
    else if((data & PER_DGT_KR) == 0)
        *my_addr += 1;
        // line wise
    else if((data & PER_DGT_TL) == 0)
        *my_addr -= SGM_MV_BYTES_PER_LINE;
    else if((data & PER_DGT_TR) == 0)
        *my_addr += SGM_MV_BYTES_PER_LINE;
    else return 0;
    
    return 1;
}

// get's a memory address
// lets user browse
// updates address to selected one
// returns with current my_addr
void SGM_MemoryViewer(Uint8 **addr, Uint32 minAddr, Uint32 maxAddr)
{
    int ret;
    Uint8 *my_addr = *addr;

    clearScreen();
    SGM_MV_draw(*addr, my_addr, maxAddr);  
    waitRelease(PER_DGT_ST | PER_DGT_TA | PER_DGT_TB | PER_DGT_TC);
      
    while(1) {
        Uint16 data, push;
        getfirstPadInput(&data, &push, NULL);
        // get inputs
        ret = SGM_MV_inputs(push, data, addr, &my_addr);
        if(ret == -1)
            break;
        else if(ret == 1) {
            if(((Uint32)my_addr) < minAddr)
                my_addr = (Uint8 *)minAddr;
            else if(((Uint32)my_addr) > maxAddr)
                my_addr = (Uint8 *)maxAddr;
                
            SGM_MV_draw(*addr, my_addr, maxAddr);
        }
        
        slSynch();
    }
// set only explicitly
//    *addr = my_addr;
    return;
}
