//                                                                                
//    Copyright (c) 2004/2005 Thomas Fuchs / The Rockin'-B, www.rockin-b.de       
//                                                                                
//    Permission is granted to anyone to use this software for any purpose        
//    and to redistribute it freely, subject to the following restrictions:       
//                                                                                
//    1.  The origin of this software must not be misrepresented. You must not    
//        claim that you wrote the original software. If you use this software    
//        in a product, an acknowledgment in the product documentation would be   
//        appreciated but is not required.                                        
//                                                                                
//    2.  ANY COMMERCIAL USE IS PROHIBITED. This includes that you must not use   
//        this software in the production of a commercial product. Only an explicit
//        permission by the author can allow you to do that.                      
//                                                                                
//    3.  Any modification applied to the software must be marked as such clearly.
//                                                                                
//    4.  This license may not be removed or altered from any distribution.       
//                                                                                
//                                                                                
//    			    NO WARRANTY                                                 
//                                                                                
//    THERE IS NO WARRANTY FOR THE PROGRAM.                                       
//    THE PROGRAM IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,               
//    EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,                 
//    THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE.                 
//    THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.
//    SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY    
//    SERVICING, REPAIR OR CORRECTION.                                            
//                                                                                
//    IN NO EVENT WILL THE COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES,         
//    INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
//    OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED   
//    TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY    
//    YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER  
//    PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE       
//    POSSIBILITY OF SUCH DAMAGES.                                                
//                                                                                

#include <SGL.H>
#include <SGM_device.h>
#include <SGM_popup.h>
#include <SGM_string.h>

extern void SGM_draw();




void SGM_copy()
{
    int ret;
    char *filename;
        // dir change & check validity
    if((aDevice->id == DEVICE_CD_DRIVE) && (isDir_CD(aDevice, aDevice->fIndex) == 1)) {
        // change directory
        ret = changeDir_CD(aDevice, aDevice->fIndex);
        SGM_draw();
        if(ret == 0)
            SGM_confirm(0xFFFF, CONFIRM_DIRCHANGE_SUCCESS, "");
        else SGM_confirm(0xFFFF, CONFIRM_DIRCHANGE_ERROR, "");
    } else if(iDevice->id == DEVICE_CD_DRIVE) 
        SGM_confirm(0xFFFF, CONFIRM_WRITE_PROTECTED, "");
    else {
        // copy by load & save
        MemoryFile mFile;
        filename = aDevice->getName(aDevice, aDevice->fIndex);
            // confirm copy
        if(SGM_confirm(PER_DGT_TA, CONFIRM_COPY, filename)) {                    
            // load
            ret = aDevice->loadFile(aDevice, aDevice->fIndex, &mFile);
            if(ret == 0) {
                // save
                    // overwrite?
                if((findFileName(iDevice, mFile.name) < 0) || (iDevice->id == DEVICE_VIRTUAL) || SGM_confirm(PER_DGT_TA, CONFIRM_OVERWRITE_A, mFile.name)) {
                    ret = iDevice->saveFile(iDevice, &mFile);
                    if(ret == 0) {
                        // update dir and screen
                        iDevice->updateDir(iDevice);
                        SGM_draw();
                        SGM_confirm(0xFFFF, CONFIRM_COPY_SUCCESS, filename);
                    } else SGM_confirm(0xFFFF, CONFIRM_SAVE_ERROR, filename);
                }    
                // free temporary file
                if(mFile.size > 0)
                    free(mFile.data);
            } else SGM_confirm(0xFFFF, CONFIRM_LOAD_ERROR, filename);
        }
    }
}

void SGM_open()
{
    int ret;
    MemoryFile mFile;
    // confirm
    if(SGM_confirm(PER_DGT_TY, CONFIRM_DECOMPRESS, aDevice->getName(aDevice, aDevice->fIndex))) {
        // load virtual drive compressed file
        ret = aDevice->loadFile(aDevice, aDevice->fIndex, &mFile);
        if(ret == 0) {
            // create a new empty virtual drive
            if(appendDevice(DEVICE_VIRTUAL) == 0) { // aDevice invalid from here
                // show new drive in unselected slot
                activeDevices[inactiveSlot] = nDevices - 1;
                // give a name:
                memcpy(iDevice->name, mFile.name, 13);
                // decompress to the new empty virtual drive
                ret = decompressFileToDevice(iDevice, &mFile);
                // display dir changes
                SGM_draw();
                if(ret != 0) {
#if 0                    
                    // remove the virtual drive
                        // delete all files
                    iDevice->format(iDevice);
                    // !!! empty drives are removed automatically
//                            // remove from list
//                        if(removeDevice(nDevices - 1) != 0)
//                            SGM_confirm(0xFFFF, CONFIRM_ERROR, "removing Device");
//                            // don't select devices out of bounds
//                        activeDevices[inactiveSlot] = nDevices-1;
                    // error message
                    SGM_confirm(0xFFFF, CONFIRM_DECOMPRESS_ERROR, aDevice->getName(aDevice, aDevice->fIndex));
#else
                    // leave already decompressed files
                    char str[20];
                    snprintf(str, 20, "Code: %x", ret);
                    SGM_confirm(0xFFFF, CONFIRM_DECOMPRESS_ERROR, str);
#endif
                } else SGM_confirm(0xFFFF, CONFIRM_DECOMPRESS_SUCCESS, aDevice->getName(aDevice, aDevice->fIndex));
            } else SGM_confirm(0xFFFF, CONFIRM_ERROR, "appending Device");
            // free compressed file memory
            if(mFile.size > 0)
                free(mFile.data);
        } else SGM_confirm(0xFFFF, CONFIRM_LOAD_ERROR, aDevice->getName(aDevice, aDevice->fIndex));
    }
}

void SGM_save()
{
    int ret;
    MemoryFile mFile;
    // confirm
    if(iDevice->id == DEVICE_CD_DRIVE)
        SGM_confirm(0xFFFF, CONFIRM_WRITE_PROTECTED, "");
    else if(SGM_confirm(PER_DGT_TZ, CONFIRM_COMPRESS, aDevice->name)) {
        // compress active slot device to file
        ret = compressDeviceToFile(aDevice, &mFile);
        if(ret == 0) {
            // save file to inactive slot device
            SGM_string("Specify file suffix", 11-3, mFile.name+3);
                // overwrite?
            if((findFileName(iDevice, mFile.name) < 0) || (iDevice->id == DEVICE_VIRTUAL) || SGM_confirm(PER_DGT_TZ, CONFIRM_OVERWRITE_Z, mFile.name)) {
                ret = iDevice->saveFile(iDevice, &mFile);
                // for display dir changes
                iDevice->updateDir(iDevice);
                if(ret == 0) {
                    // remove device IF VIRTUAL
                    if(aDevice->id == DEVICE_VIRTUAL) {
                        // delete all files
                        aDevice->format(aDevice);
                        // reset name
                        memcpy(aDevice->name, deviceNames[DEVICE_VIRTUAL], 17);
                        // removed automatically
//                        // remove from list
//                        if(removeDevice(activeDevices[activeSlot]) != 0)
//                            SGM_confirm(0xFFFF, CONFIRM_ERROR, "removing Device");
//                        // has the inactive drive shifted?
//                        if(activeDevices[activeSlot] <= activeDevices[inactiveSlot])
//                            activeDevices[inactiveSlot]--;
//                        // !!! always keep one empty virtual device
//                        if(nDevices <= DEVICE_VIRTUAL) {
//                            if(appendDevice(DEVICE_VIRTUAL) == 0) {
//                                activeDevices[activeSlot] = nDevices-1;
//                            } else SGM_confirm(0xFFFF, CONFIRM_ERROR, "appending Device");
//                        }
//                        // don't select devices out of bounds
//                        if(activeDevices[activeSlot] >= nDevices)
//                            activeDevices[activeSlot]--;
                    }
                    SGM_draw();
                    SGM_confirm(0xFFFF, CONFIRM_COMPRESS_SUCCESS, mFile.name);
                } else {
                    // display dir changes
                    SGM_draw();
                    SGM_confirm(0xFFFF, CONFIRM_SAVE_ERROR, mFile.name);
                }
            }    
        } else {
            SGM_draw(); // to show if there is a memory leak, but there shouldn't be any
            SGM_confirm(0xFFFF, CONFIRM_COMPRESS_ERROR, "");
        }    
        // free temporary file data
        if(mFile.size > 0)
            free(mFile.data);
    }
}


void SGM_delete()
{
    int ret;
    char *filename;
        // check validity
    if(aDevice->id == DEVICE_CD_DRIVE)
        SGM_confirm(0xFFFF, CONFIRM_WRITE_PROTECTED, "");
    else {
        filename = aDevice->getName(aDevice, aDevice->fIndex);
            // confirm delete
        if(SGM_confirm(PER_DGT_TB, CONFIRM_DELETE, filename)) {  
            ret = aDevice->deleteFile(aDevice, aDevice->fIndex);
            if(ret == 0) {
                // update dir and screen
                aDevice->updateDir(aDevice);
                // update file index (cursor)
                if(aDevice->getFileNum(aDevice) == 0)
                    aDevice->fIndex = 0;
                else if(aDevice->fIndex >= aDevice->getFileNum(aDevice))
                    aDevice->fIndex = aDevice->getFileNum(aDevice)-1;
                SGM_draw();
                SGM_confirm(0xFFFF, CONFIRM_DELETE_SUCCESS, "");    // filename is no more available
            } else SGM_confirm(0xFFFF, CONFIRM_DELETE_ERROR, filename);
        } 
    }                 
}

void SGM_format()
{
    int ret;
    // check validity
    if(aDevice->id == DEVICE_CD_DRIVE)
        SGM_confirm(0xFFFF, CONFIRM_WRITE_PROTECTED, "");
    else {
            // confirm format
        if(SGM_confirm(PER_DGT_TC, CONFIRM_FORMAT, aDevice->name)) {  
            ret = aDevice->format(aDevice);
            if(ret == 0) {
                // update dir and screen
                aDevice->updateDir(aDevice);
//                    // update file index (cursor)
//                    if(aDevice->getFileNum(aDevice) == 0)
//                        aDevice->fIndex = 0;
//                    else if(aDevice->fIndex >= aDevice->getFileNum(aDevice))
//                        aDevice->fIndex = aDevice->getFileNum(aDevice)-1;
                SGM_draw();
                SGM_confirm(0xFFFF, CONFIRM_FORMAT_SUCCESS, aDevice->name);
            } else SGM_confirm(0xFFFF, CONFIRM_FORMAT_ERROR, aDevice->name);
        } 
    }                 
}
