//                                                                                
//    Copyright (c) 2004/2005 Thomas Fuchs / The Rockin'-B, www.rockin-b.de       
//                                                                                
//    Permission is granted to anyone to use this software for any purpose        
//    and to redistribute it freely, subject to the following restrictions:       
//                                                                                
//    1.  The origin of this software must not be misrepresented. You must not    
//        claim that you wrote the original software. If you use this software    
//        in a product, an acknowledgment in the product documentation would be   
//        appreciated but is not required.                                        
//                                                                                
//    2.  ANY COMMERCIAL USE IS PROHIBITED. This includes that you must not use   
//        this software in the production of a commercial product. Only an explicit
//        permission by the author can allow you to do that.                      
//                                                                                
//    3.  Any modification applied to the software must be marked as such clearly.
//                                                                                
//    4.  This license may not be removed or altered from any distribution.       
//                                                                                
//                                                                                
//    			    NO WARRANTY                                                 
//                                                                                
//    THERE IS NO WARRANTY FOR THE PROGRAM.                                       
//    THE PROGRAM IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,               
//    EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,                 
//    THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE.                 
//    THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.
//    SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY    
//    SERVICING, REPAIR OR CORRECTION.                                            
//                                                                                
//    IN NO EVENT WILL THE COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES,         
//    INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
//    OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED   
//    TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY    
//    YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER  
//    PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE       
//    POSSIBILITY OF SUCH DAMAGES.                                                
//                                                                                

#ifndef HAVE_SGM_DEF_H
#define HAVE_SGM_DEF_H

#include <SGL.H>
#include <SEGA_BUP.H>   // used here and there
#include <SEGA_CDC.H>   // used here and there
#include <RB_CartRAM.h>
#include <RB_were_here.h>

// with BGET memory allocator, you can use every possible
// RAM area in Saturn
// -> increase stability and range of propper function, 
// -> use RAM cart and increase compression quality
//#define USE_BGET      // global



#ifdef USE_BGET
#   include <bget.h>
#   ifdef DEBUG_BGET
extern void *dbg_bget(bufsize size);
extern void *dbg_bgetz(bufsize size);
extern void *dbg_bgetr(void *buffer, bufsize newsize);
extern void	dbg_brel(void *buf);
#       define malloc(sz)          dbg_bget(sz)
#       define calloc(sz, n)       dbg_bgetz(sz*n)
#       define realloc(b, sz)      dbg_bgetr(b, sz)
#       define free(b)             dbg_brel(b)
#   else
#       define malloc(sz)          bget(sz)
#       define calloc(sz, n)       bgetz(sz*n)
#       define realloc(b, sz)      bgetr(b, sz)
#       define free(b)             brel(b)
#   endif
extern Uint32 _bend;
#endif

// without BGET, SEGA_MEM is used to make BZip2 use low work RAM
// -> very limited function and stability
#define LOW_MEM_USED        (0)
#define LOW_MEM_START       (0x200000 + LOW_MEM_USED)
#define LOW_MEM_SIZE        (0x100000 - LOW_MEM_USED)


typedef struct MEMORY_FILE {
    char name[12+1]; // CD filenames must fit in, too
    char comment[10+1];
    Uint8 language, minute, hour, weekday, day, month;
    Uint16 year;
    Uint32 size;
    void *data;
} MemoryFile;

// these are as well device ids
// as they are indices to the device array
// all subsequent devices are virtual drives
typedef enum DEVICE_ID {
    DEVICE_SYSTEM_MEMORY,
    DEVICE_CARTRIDGE_MEMORY,
    DEVICE_EXTERNAL,
    DEVICE_CD_DRIVE,
    DEVICE_VIRTUAL // all cirtual drives 
} DeviceId;


typedef struct STORAGE_DEVICE {
    char name[17];
    DeviceId id;
    Uint16 fIndex;  // selected file
    void *data;     // private data
    // file management function
    int (*loadFile)(struct STORAGE_DEVICE *sDevice, Uint16 fIndex, MemoryFile *mFile);
    int (*saveFile)(struct STORAGE_DEVICE *sDevice, MemoryFile *mFile);
    int (*deleteFile)(struct STORAGE_DEVICE *sDevice, Uint16 fIndex);
    int (*format)(struct STORAGE_DEVICE *sDevice);
    // other
    int (*updateDir)(struct STORAGE_DEVICE *sDevice);
    int (*getFileNum)(struct STORAGE_DEVICE *sDevice);
    int (*getFreeBytes)(struct STORAGE_DEVICE *sDevice);
    char *(*getName)(struct STORAGE_DEVICE *sDevice, Uint16 fIndex);
    char *(*getComment)(struct STORAGE_DEVICE *sDevice, Uint16 fIndex);
    int (*getSize)(struct STORAGE_DEVICE *sDevice, Uint16 fIndex);
    int (*getDate)(struct STORAGE_DEVICE *sDevice, Uint16 fIndex, MemoryFile *mFile);
} StorageDevice;

typedef struct VIRTUAL_DRIVE {
    Uint32 nFiles;
    MemoryFile *mFile;
} VirtualDrive;



#ifdef __cplusplus
extern "C" {
#endif

Sint16 CartID;
Sint8 getfirstPadInput(Uint16 *data, Uint16 *push, Uint16 *pull);
void checkExitHere(Uint16 data);
void clearScreen();
void waitRelease(Uint16 buttons);
int memoryEqual(Uint8 *a, Uint8 *b, Uint32 n);

#ifdef __cplusplus
}
#endif

#endif //HAVE_SGM_DEF_H
