//                                                                                
//    Copyright (c) 2004/2005 Thomas Fuchs / The Rockin'-B, www.rockin-b.de       
//                                                                                
//    Permission is granted to anyone to use this software for any purpose        
//    and to redistribute it freely, subject to the following restrictions:       
//                                                                                
//    1.  The origin of this software must not be misrepresented. You must not    
//        claim that you wrote the original software. If you use this software    
//        in a product, an acknowledgment in the product documentation would be   
//        appreciated but is not required.                                        
//                                                                                
//    2.  ANY COMMERCIAL USE IS PROHIBITED. This includes that you must not use   
//        this software in the production of a commercial product. Only an explicit
//        permission by the author can allow you to do that.                      
//                                                                                
//    3.  Any modification applied to the software must be marked as such clearly.
//                                                                                
//    4.  This license may not be removed or altered from any distribution.       
//                                                                                
//                                                                                
//    			    NO WARRANTY                                                 
//                                                                                
//    THERE IS NO WARRANTY FOR THE PROGRAM.                                       
//    THE PROGRAM IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,               
//    EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,                 
//    THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE.                 
//    THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.
//    SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY    
//    SERVICING, REPAIR OR CORRECTION.                                            
//                                                                                
//    IN NO EVENT WILL THE COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES,         
//    INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
//    OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED   
//    TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY    
//    YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER  
//    PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE       
//    POSSIBILITY OF SUCH DAMAGES.                                                
//               
                                                                 
#include <SGM_def.h>
#include <SGM_device.h>
#include <SGM_popup.h>

char *deviceNames[] = {
    "System Memory   ",
    "Cartridge Memory",
    "Serial Device   ",
    "CD Drive        ",
    "Virtual Drive   ",
};
char filename_NULL[] = "            ";
char comment_NULL[] = "           ";


// which device is active in which slot
Uint8 activeSlot = 0, inactiveSlot = 1;
Uint8 activeDevices[2] = {DEVICE_CD_DRIVE, DEVICE_SYSTEM_MEMORY};
// device list
Uint8 nDevices = 0;
StorageDevice *device = NULL;

/****************************************************************************************************/
/* file management functions ************************************************************************/
/****************************************************************************************************/

int stringsEqual(char *a, char *b)
{
    int o = 0;
    while(*(a+o) == *(b+o)) {
        // equal
        if(*(a+o) == '\0')
            return 1;
            
        o++;
    }
    return 0;
}

// returns first file with same filename
// or -1 if not found
int findFileName(struct STORAGE_DEVICE *sDevice, char *filename)
{
    int i, n;
    char *s;
    // check args
    if((sDevice == NULL) || (filename == NULL))
        return -1;
    // search file name
    n = sDevice->getFileNum(sDevice);
    for(i = 0; i < n; i++) {
        if(stringsEqual(filename, sDevice->getName(sDevice, i)))
            return i;
    }
    return -1;  // not found
}

/****************************************************************************************************/
/* device array handling ****************************************************************************/
/****************************************************************************************************/

int createDevice(struct STORAGE_DEVICE *sDevice, DeviceId id)
{
    if(id <= 2){
        sDevice->id = id;
        sDevice->data = NULL;
        sDevice->loadFile = loadFile_Backup;
        sDevice->saveFile = saveFile_Backup;
        sDevice->deleteFile = deleteFile_Backup;
        sDevice->format = format_Backup;
        sDevice->updateDir = updateDir_Backup;
        sDevice->getFileNum = getFileNum_Backup;
        sDevice->getFreeBytes = getFreeBytes_Backup;
        sDevice->getName = getName_Backup;
        sDevice->getComment = getComment_Backup;
        sDevice->getSize = getSize_Backup;
        sDevice->getDate = getDate_Backup;
    } else if(id == DEVICE_CD_DRIVE) {
        sDevice->id = id;
        sDevice->data = NULL;
        sDevice->loadFile = loadFile_CD;
        sDevice->saveFile = NULL;
        sDevice->deleteFile = NULL;
        sDevice->format = NULL;
        sDevice->updateDir = updateDir_CD;
        sDevice->getFileNum = getFileNum_CD;
        sDevice->getFreeBytes = getFreeBytes_CD;
        sDevice->getName = getName_CD;
        sDevice->getComment = getComment_CD;
        sDevice->getSize = getSize_CD;
        sDevice->getDate = getDate_CD;
    } else if(id == DEVICE_VIRTUAL) {
        sDevice->id = id;
        sDevice->data = (VirtualDrive *)calloc(1, sizeof(VirtualDrive));
        if(sDevice->data == NULL)
            return -2;
        sDevice->loadFile = loadFile_Virtual;
        sDevice->saveFile = saveFile_Virtual;
        sDevice->deleteFile = deleteFile_Virtual;
        sDevice->format = format_Virtual;
        sDevice->updateDir = updateDir_Virtual;
        sDevice->getFileNum = getFileNum_Virtual;
        sDevice->getFreeBytes = getFreeBytes_Virtual;
        sDevice->getName = getName_Virtual;
        sDevice->getComment = getComment_Virtual;
        sDevice->getSize = getSize_Virtual;
        sDevice->getDate = getDate_Virtual;
    } else return -1;

//    sDevice->name = deviceNames[id];
    memcpy(sDevice->name, deviceNames[id], 17);
    sDevice->fIndex = 0;
    return 0;
}

int appendDevice(DeviceId id)
{
    StorageDevice *sDevice;
    int ret;
    
    if(nDevices == 0)
        sDevice = (StorageDevice *)malloc(sizeof(StorageDevice));
    else sDevice = (StorageDevice *)realloc(device, (nDevices + 1) * sizeof(StorageDevice));

    if(sDevice == NULL)
        return -2;
        
    device = sDevice;
    ret = createDevice(device + nDevices, id);
    if(ret != 0) {
        // must TRY to free that device
        sDevice = (StorageDevice *)realloc(device, (nDevices) * sizeof(StorageDevice));
        if(sDevice != NULL)
            device = sDevice;
        return ret;
    }    
    nDevices++;

    return 0;
}

int removeDevice(Uint8 index)
{
    int i;
    StorageDevice *sDevice;
    // check args
    if(index >= nDevices)
        return -1;
    // free private data allocated in create
    if((device + index)->id == DEVICE_VIRTUAL) {
        // can assume that all files are deleted,
        // as well as the MemoryFile array
        free((device + index)->data);
    }    
    // shift entries
    for(i = index+1; i < nDevices; i++)
        memcpy(device + i-1, device + i, sizeof(StorageDevice));

    sDevice = (StorageDevice *)realloc(device, (nDevices - 1) * sizeof(StorageDevice));
    if(sDevice == NULL) {
        // last device doubled, 
        // so I MUST deactivate it
        (device + nDevices - 1)->data = NULL;
        return -2;
    }    
    
    device = sDevice;
    nDevices--;
    return 0;
}

// - always keep one empty virtual drive
// - unneeded empty virtual drives
// - correct slot devices, if neccessary
int checkVirtualDevices()
{
    int i, nEmpty = 0;
    
    for(i = DEVICE_VIRTUAL; i < nDevices; i++) {
        // count empty virtual drives
        if((device + i)->getFileNum(device + i) == 0)
            nEmpty++;
        // remove all unneeded empty drives
        if(nEmpty > 1) {
            if(removeDevice(i) != 0)
                SGM_confirm(0xFFFF, CONFIRM_ERROR, "removing Device");
            // correct slots
            if(activeDevices[activeSlot] >= i)
                activeDevices[activeSlot]--;
            if(activeDevices[inactiveSlot] >= i)
                activeDevices[inactiveSlot]--;
        }
    }
    // always keep one empty virtual drive
    if(nEmpty == 0) {
        if(appendDevice(DEVICE_VIRTUAL) != 0)
            SGM_confirm(0xFFFF, CONFIRM_ERROR, "appending Device");
    }
    // don't select the same drive
    if(activeDevices[inactiveSlot] == activeDevices[activeSlot])
        activeDevices[inactiveSlot]--;
}

int init_Devices()
{
    // allocate
//    device = (StorageDevice *)malloc(5 * sizeof(StorageDevice));
//    if(device == NULL)
//        return -2;
//    nDevices = 5;
    // set
    appendDevice(DEVICE_SYSTEM_MEMORY);
    appendDevice(DEVICE_CARTRIDGE_MEMORY);
    appendDevice(DEVICE_EXTERNAL);
    appendDevice(DEVICE_CD_DRIVE);
    appendDevice(DEVICE_VIRTUAL);
    
    return 0;
}

