//                                                                                
//    Copyright (c) 2004/2005 Thomas Fuchs / The Rockin'-B, www.rockin-b.de       
//                                                                                
//    Permission is granted to anyone to use this software for any purpose        
//    and to redistribute it freely, subject to the following restrictions:       
//                                                                                
//    1.  The origin of this software must not be misrepresented. You must not    
//        claim that you wrote the original software. If you use this software    
//        in a product, an acknowledgment in the product documentation would be   
//        appreciated but is not required.                                        
//                                                                                
//    2.  ANY COMMERCIAL USE IS PROHIBITED. This includes that you must not use   
//        this software in the production of a commercial product. Only an explicit
//        permission by the author can allow you to do that.                      
//                                                                                
//    3.  Any modification applied to the software must be marked as such clearly.
//                                                                                
//    4.  This license may not be removed or altered from any distribution.       
//                                                                                
//                                                                                
//    			    NO WARRANTY                                                 
//                                                                                
//    THERE IS NO WARRANTY FOR THE PROGRAM.                                       
//    THE PROGRAM IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,               
//    EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,                 
//    THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE.                 
//    THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.
//    SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY    
//    SERVICING, REPAIR OR CORRECTION.                                            
//                                                                                
//    IN NO EVENT WILL THE COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES,         
//    INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
//    OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED   
//    TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY    
//    YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER  
//    PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE       
//    POSSIBILITY OF SUCH DAMAGES.                                                
//                                                                                

#include <SGM_def.h>
#include <SGM_device.h>

/****************************************************************************************************/
/* CD access stuff **********************************************************************************/
/****************************************************************************************************/

#include <SEGA_GFS.H>
#define   OPEN_MAX  (Sint32)5
#define   DIR_MAX   (Sint32)25   
 
// keep this info global
GfsDirTbl gfsDirTbl;
GfsDirName gfsDirName[DIR_MAX];
Uint32 gfsLibWork[GFS_WORK_SIZE(OPEN_MAX)/sizeof(Uint32)];     
// number of files in current directory
Sint32 gfsDirN;

void init_CD()
{
    GFS_DIRTBL_TYPE(&gfsDirTbl) = GFS_DIR_NAME;
    GFS_DIRTBL_DIRNAME(&gfsDirTbl) = gfsDirName;
    GFS_DIRTBL_NDIR(&gfsDirTbl) = DIR_MAX;
    gfsDirN = GFS_Init(OPEN_MAX, gfsLibWork, &gfsDirTbl);
}

// taken from somewhere ???
#define SYS_CDINIT1(i) \
     ((**(void(**)(int))0x60002dc)(i))

#define SYS_CDINIT2() \
     ((**(void(**)(void))0x600029c)())

int CdUnlock (void)
{
     Sint32 ret;
     CdcStat stat;
     volatile int delay;

     SYS_CDINIT1(3);

     SYS_CDINIT2();

     do {
          for(delay = 100000; delay; delay--);

          ret = CDC_GetCurStat(&stat);

     } while ((ret != 0) || (CDC_STAT_STATUS(&stat) == 0xff));

     return (int) CDC_STAT_STATUS(&stat);
}

/****************************************************************************************************/
/* CD Drive interface *******************************************************************************/
/****************************************************************************************************/

int updateDir_CD(struct STORAGE_DEVICE *sDevice)
{
    CdUnlock();
    init_CD();
    
    return 0;
}

int getFileNum_CD(struct STORAGE_DEVICE *sDevice)
{
    return gfsDirN;
}

int getFreeBytes_CD(struct STORAGE_DEVICE *sDevice)
{
    return 0;
}

char *getName_CD(struct STORAGE_DEVICE *sDevice, Uint16 fIndex)
{
    // check args
    if(fIndex >= gfsDirN)
        return filename_NULL;

    return GFS_DIR_FNAME(&gfsDirName[fIndex]);
}

char *getComment_CD(struct STORAGE_DEVICE *sDevice, Uint16 fIndex)
{
    // check args
    if(fIndex >= gfsDirN)
        return comment_NULL;
        
    return comment_NULL;
}

int getSize_CD(struct STORAGE_DEVICE *sDevice, Uint16 fIndex)
{
    // check args
    if(fIndex >= gfsDirN)
        return -1;

    return GFS_DIR_SIZE(&gfsDirName[fIndex]);
}

int getDate_CD(struct STORAGE_DEVICE *sDevice, Uint16 fIndex, MemoryFile *mFile)
{
    // check args
    if((fIndex >= gfsDirN) || (mFile == NULL))
        return -1;

    mFile->year = 1980;
    mFile->month = 6;
    mFile->day = 6;
    mFile->weekday = 0;
    mFile->hour = 0;
    mFile->minute = 0;

    return 0;
}

// how to get date information?
int loadFile_CD(struct STORAGE_DEVICE *sDevice, Uint16 fIndex, MemoryFile *mFile)
{
    Sint32 size;
    
    // check args
    if((mFile == NULL) || (fIndex >= gfsDirN))
        return -1;
    mFile->size = 0;
    // allocate memory
    size = GFS_DIR_SIZE(&gfsDirName[fIndex]);
    // handle empty files
    if(size > 0) {
        mFile->data = (void *)malloc(size);   
        if(mFile->data == NULL)
            return -2;
        // load data
      	size = GFS_Load(fIndex, 0, (Uint32 *)mFile->data, size);
        if(size <= 0) {
            free(mFile->data);
            return -3;
        }
    }    
    // write further attributes
    mFile->size = size;
        // assuming that mFile->name is longer or of equal length as 8.3
#if 0
    memcpy(mFile->name, GFS_DIR_FNAME(&gfsDirName[fIndex]), (8+3+1+1)*sizeof(char));
#else
    {   // skip '.'
        int s, t;
        char *str = GFS_DIR_FNAME(&gfsDirName[fIndex]);
        for (s = 0, t = 0; (*(str+s) != '\0') && (s < (8+1+3+1)); s++) {
            if(*(str+s) != '.')
                *(mFile->name+t++) = *(str+s);
        }
        *(mFile->name+t) = '\0';
    }
#endif
    mFile->comment[0] = '\0';
    mFile->language = BUP_DEUTSCH;
    getDate_CD(sDevice, fIndex, mFile);    
//    mFile->year = 1980;
//    mFile->month = 6;
//    mFile->day = 6;
//    mFile->weekday = 0;
//    mFile->hour = 0;
//    mFile->minute = 0;
    
    return 0;
}

int isDir_CD(struct STORAGE_DEVICE *sDevice, Uint16 fIndex)
{
    if((sDevice == NULL) || (fIndex >= sDevice->getFileNum(sDevice)))
        return -1;

    return ((GFS_DIR_ATR(&gfsDirName[fIndex]) & GFS_ATR_DIR) != 0);
}

int changeDir_CD(struct STORAGE_DEVICE *sDevice, Uint16 fIndex)
{
    Sint32 fid, set;
    // check args
    if((sDevice == NULL) || (fIndex >= sDevice->getFileNum(sDevice)) || (isDir_CD(sDevice, fIndex) != 1))
        return -1;
    // change dir
    fid = GFS_NameToId(GFS_DIR_FNAME(&gfsDirName[fIndex]));
    gfsDirN = GFS_LoadDir(fid, &gfsDirTbl);
    set = GFS_SetDir(&gfsDirTbl);
    sDevice->fIndex = 0;
    if((fid >= 0) && (gfsDirN >= 0) && (set >= 0))
        return 0;
    else return -3;
}

