//                                                                                
//    Copyright (c) 2004/2005 Thomas Fuchs / The Rockin'-B, www.rockin-b.de       
//                                                                                
//    Permission is granted to anyone to use this software for any purpose        
//    and to redistribute it freely, subject to the following restrictions:       
//                                                                                
//    1.  The origin of this software must not be misrepresented. You must not    
//        claim that you wrote the original software. If you use this software    
//        in a product, an acknowledgment in the product documentation would be   
//        appreciated but is not required.                                        
//                                                                                
//    2.  ANY COMMERCIAL USE IS PROHIBITED. This includes that you must not use   
//        this software in the production of a commercial product. Only an explicit
//        permission by the author can allow you to do that.                      
//                                                                                
//    3.  Any modification applied to the software must be marked as such clearly.
//                                                                                
//    4.  This license may not be removed or altered from any distribution.       
//                                                                                
//                                                                                
//    			    NO WARRANTY                                                 
//                                                                                
//    THERE IS NO WARRANTY FOR THE PROGRAM.                                       
//    THE PROGRAM IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,               
//    EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,                 
//    THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE.                 
//    THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.
//    SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY    
//    SERVICING, REPAIR OR CORRECTION.                                            
//                                                                                
//    IN NO EVENT WILL THE COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES,         
//    INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
//    OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED   
//    TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY    
//    YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER  
//    PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE       
//    POSSIBILITY OF SUCH DAMAGES.                                                
//                                                                                

#include <SGM_def.h>
#include <SGM_device.h>

/****************************************************************************************************/
/* backup RAM stuff *********************************************************************************/
/****************************************************************************************************/

#include <sega_bup.h>

#define BUP_DIR_SIZE 400
BupConfig bupConfig[3];
BupStat bupStat[3];
BupDir bupDir[3][BUP_DIR_SIZE];
Sint32 bupDirN[3];

/* This is to init backup library
 * Note that SGL demo got a terrible error 
 * that can cause to damage controller access during gameplay:
 * Always keep these two array as global variables. */
// don't take smaller lengths !!!
Uint32  BackUpRamWork[2048];
Uint8   BackUpLibSpace[16384];


int init_BackUp()
{
    int i;
    Sint32 ret;
    
	slResetDisable();
	BUP_Init((Uint32 *)BackUpLibSpace, BackUpRamWork, bupConfig);
	slResetEnable();
	// read directories of all backup devices devices
	for(i = 0; i < 3; i++) {
        ret = BUP_Dir(i, (Uint8 *)"", BUP_DIR_SIZE, bupDir[i]);
        if(ret < 0)
            return -3;
        // remember number of files
        bupDirN[i] = ret;
        // device status
        ret = BUP_Stat(i, 0, bupStat + i);
        if(ret != 0) {
            memset(bupStat + i, 0, sizeof(BupStat));
            return -3;
        }
    }
}

///* taken from sgl sample 13-1 */
//Sint32	BackUpWrite(Uint32 device, BupDir *dir, Uint8 *data, Uint8 sw)
//{
//	Sint32		ret;
//	SmpcDateTime	*time;
//	BupDate		date;
//	
//	if(!dir->date){
//		time = &(Smpc_Status->rtc);
//		date.year  = (Uint8 )( slDec2Hex((Uint32)time->year)-1980 ); /* Modify K.T */
//		date.month = (Uint8 )( time->month & 0x0f);
//		date.week  = (Uint8 )( time->month >> 4  );
//		date.day   = (Uint8 )( slDec2Hex((Uint32)time->date));   /* Modify K.T */
//		date.time  = (Uint8 )( slDec2Hex((Uint32)time->hour));   /* Modify K.T */
//		date.min   = (Uint8 )( slDec2Hex((Uint32)time->minute)); /* Modify K.T */
//		dir->date = BUP_SetDate(&date);
//	}
//	slResetDisable();
//	ret = BUP_Write(device,dir,data,sw);
//	slResetEnable();
//
//	return(ret);
//}


/****************************************************************************************************/
/* Backup RAM interface *****************************************************************************/
/****************************************************************************************************/

int updateDir_Backup(struct STORAGE_DEVICE *sDevice)
{
    Sint32 ret;
    
    // check args
    if((sDevice == NULL) || (sDevice->id > 2))
        return -1;
    // read directory info
    ret = BUP_Dir(sDevice->id, (Uint8 *)"", BUP_DIR_SIZE, bupDir[sDevice->id]);
    if(ret < 0)
        return -3;
    // remember number of files
    bupDirN[sDevice->id] = ret;
    // device status
    ret = BUP_Stat(sDevice->id, 0, bupStat + sDevice->id);
    if(ret != 0) {
        memset(bupStat + sDevice->id, 0, sizeof(BupStat));
        return -3;
    }

    return 0;
}

int getFileNum_Backup(struct STORAGE_DEVICE *sDevice)
{
    // check args
    if((sDevice == NULL) || (sDevice->id > 2))
        return -1;

    return bupDirN[sDevice->id];
}

int getFreeBytes_Backup(struct STORAGE_DEVICE *sDevice)
{
    // check args
    if((sDevice == NULL) || (sDevice->id > 2))
        return -1;

    return bupStat[sDevice->id].freesize;
}

char *getName_Backup(struct STORAGE_DEVICE *sDevice, Uint16 fIndex)
{
    // check args
    if((sDevice == NULL) || (sDevice->id > 2) || (fIndex >= bupDirN[sDevice->id]))
        return filename_NULL;

    return bupDir[sDevice->id][fIndex].filename;
}

char *getComment_Backup(struct STORAGE_DEVICE *sDevice, Uint16 fIndex)
{
    // check args
    if((sDevice == NULL) || (sDevice->id > 2) || (fIndex >= bupDirN[sDevice->id]))
        return comment_NULL;
        
    return bupDir[sDevice->id][fIndex].comment;
}

int getSize_Backup(struct STORAGE_DEVICE *sDevice, Uint16 fIndex)
{
    // check args
    if((sDevice == NULL) || (sDevice->id > 2) || (fIndex >= bupDirN[sDevice->id]))
        return -1;

    return bupDir[sDevice->id][fIndex].datasize;
}

int getDate_Backup(struct STORAGE_DEVICE *sDevice, Uint16 fIndex, MemoryFile *mFile)
{
    BupDate bupDate;
    // check args
    if((sDevice == NULL) || (sDevice->id > 2) || (fIndex >= bupDirN[sDevice->id]) || (mFile == NULL))
        return -1;
    // get date
    BUP_GetDate(bupDir[sDevice->id][fIndex].date, &bupDate);
    mFile->year = 1980 + (Uint16)bupDate.year;
    mFile->month = bupDate.month;
    mFile->day = bupDate.day;
    mFile->weekday = bupDate.week;
    mFile->hour = bupDate.time;
    mFile->minute = bupDate.min;

    return 0;
}

int loadFile_Backup(struct STORAGE_DEVICE *sDevice, Uint16 fIndex, MemoryFile *mFile)
{
    Uint32 size;
    Sint32 ret;

    // check args
    if((mFile == NULL) || (sDevice == NULL) || (sDevice->id > 2) || (fIndex >= bupDirN[sDevice->id]))
        return -1;
    mFile->size = 0;
    // allocate memory
    size = bupDir[sDevice->id][fIndex].datasize;
    // handle empty files
    if(size > 0) {
        mFile->data = (void *)malloc(size);
        if(mFile->data == NULL)
            return -2;
        // load data
    	slResetDisable();
    	ret = BUP_Read(sDevice->id, bupDir[sDevice->id][fIndex].filename, mFile->data);
    	slResetEnable();
        if(ret != 0) {
            free(mFile->data);
            return -3;
        }
    }        
    // write further attributes
    mFile->size = size;
    memcpy(mFile->name, bupDir[sDevice->id][fIndex].filename, 12);
    memcpy(mFile->comment, bupDir[sDevice->id][fIndex].comment, 11);
    mFile->language = bupDir[sDevice->id][fIndex].language;

    getDate_Backup(sDevice, fIndex, mFile);
//    BUP_GetDate(bupDir[sDevice->id][fIndex].date, &bupDate);
//    mFile->year = 1980 + (Uint16)bupDate.year;
//    mFile->month = bupDate.month;
//    mFile->day = bupDate.day;
//    mFile->weekday = bupDate.week;
//    mFile->hour = bupDate.time;
//    mFile->minute = bupDate.min;
    
    return 0;
}

// overwrites files with same name
int saveFile_Backup(struct STORAGE_DEVICE *sDevice, MemoryFile *mFile)
{
    Sint32 ret;
	BupDir bupDir; // masks global variable...
	BupDate bupDate;

    // check args (and size <= 256K)
    if((mFile == NULL) || (mFile->size > (256*1024)) || (sDevice == NULL) || (sDevice->id > 2))
        return -1;
	// set dir information
    memcpy(bupDir.filename, mFile->name, 12);
    memcpy(bupDir.comment, mFile->comment, 11);
    bupDir.language = mFile->language;
    bupDate.year = (Uint8)(mFile->year - 1980);
    bupDate.month = mFile->month;
    bupDate.day = mFile->day;
    bupDate.week = mFile->weekday;
    bupDate.time = mFile->hour;
    bupDate.min = mFile->minute;
    bupDir.date = BUP_SetDate(&bupDate);
	bupDir.datasize = mFile->size;
	// write data
	slResetDisable();
	ret = BUP_Write(sDevice->id, &bupDir, mFile->data, OFF);
	slResetEnable();
    if(ret != 0)
        return -3;
    // verify data
	ret = BUP_Verify(sDevice->id, bupDir.filename, mFile->data);
    if(ret != 0)
        return -3;
    
    return 0;
}

int deleteFile_Backup(struct STORAGE_DEVICE *sDevice, Uint16 fIndex)
{
	Sint32	ret;

    // check args
    if((sDevice == NULL) || (sDevice->id > 2) || (fIndex >= bupDirN[sDevice->id]))
        return -1;
    // delete
	slResetDisable();
	ret = BUP_Delete(sDevice->id, bupDir[sDevice->id][fIndex].filename);
	slResetEnable();
    if(ret != 0)
        return -3;
        
    return 0;
}

int format_Backup(struct STORAGE_DEVICE *sDevice)
{
    Sint32 ret;
    // check args
    if((sDevice == NULL) || (sDevice->id > 2))
        return -1;
    // format
    slResetDisable();
    ret = BUP_Format(sDevice->id);
    slResetEnable();
    
    if(ret != 0)
        return -3;

    sDevice->fIndex = 0;

	return 0;
}

