//                                                                                
//    Copyright (c) 2004/2005 Thomas Fuchs / The Rockin'-B, www.rockin-b.de       
//                                                                                
//    Permission is granted to anyone to use this software for any purpose        
//    and to redistribute it freely, subject to the following restrictions:       
//                                                                                
//    1.  The origin of this software must not be misrepresented. You must not    
//        claim that you wrote the original software. If you use this software    
//        in a product, an acknowledgment in the product documentation would be   
//        appreciated but is not required.                                        
//                                                                                
//    2.  ANY COMMERCIAL USE IS PROHIBITED. This includes that you must not use   
//        this software in the production of a commercial product. Only an explicit
//        permission by the author can allow you to do that.                      
//                                                                                
//    3.  Any modification applied to the software must be marked as such clearly.
//                                                                                
//    4.  This license may not be removed or altered from any distribution.       
//                                                                                
//                                                                                
//    			    NO WARRANTY                                                 
//                                                                                
//    THERE IS NO WARRANTY FOR THE PROGRAM.                                       
//    THE PROGRAM IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,               
//    EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,                 
//    THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE.                 
//    THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.
//    SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY    
//    SERVICING, REPAIR OR CORRECTION.                                            
//                                                                                
//    IN NO EVENT WILL THE COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES,         
//    INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
//    OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED   
//    TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY    
//    YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER  
//    PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE       
//    POSSIBILITY OF SUCH DAMAGES.                                                
//                                                                                

#include <SGM_def.h>
#include <SGM_device.h>

/****************************************************************************************************/
/* Virtual Drive interface **************************************************************************/
/****************************************************************************************************/

int updateDir_Virtual(struct STORAGE_DEVICE *sDevice)
{
    // always up to date
    return 0;
}

int getFileNum_Virtual(struct STORAGE_DEVICE *sDevice)
{
    // check args
    if((sDevice == NULL) || (sDevice->data == NULL))
        return -1;
        
    return ((VirtualDrive *)sDevice->data)->nFiles;
}

int getFreeBytes_Virtual(struct STORAGE_DEVICE *sDevice)
{
    // check args
    if((sDevice == NULL) || (sDevice->data == NULL))
        return -1;
        
    return 256*1024;
}

char *getName_Virtual(struct STORAGE_DEVICE *sDevice, Uint16 fIndex)
{
    // check args
    if((sDevice == NULL) || (sDevice->data == NULL) || (fIndex >= ((VirtualDrive *)sDevice->data)->nFiles))
        return filename_NULL;

    return (((VirtualDrive *)sDevice->data)->mFile + fIndex)->name;
}

char *getComment_Virtual(struct STORAGE_DEVICE *sDevice, Uint16 fIndex)
{
    // check args
    if((sDevice == NULL) || (sDevice->data == NULL) || (fIndex >= ((VirtualDrive *)sDevice->data)->nFiles))
        return comment_NULL;

    return (((VirtualDrive *)sDevice->data)->mFile + fIndex)->comment;
}

int getSize_Virtual(struct STORAGE_DEVICE *sDevice, Uint16 fIndex)
{
    // check args
    if((sDevice == NULL) || (sDevice->data == NULL) || (fIndex >= ((VirtualDrive *)sDevice->data)->nFiles))
        return -1;

    return (((VirtualDrive *)sDevice->data)->mFile + fIndex)->size;
}

int getDate_Virtual(struct STORAGE_DEVICE *sDevice, Uint16 fIndex, MemoryFile *mFile)
{
    // check args
    if((sDevice == NULL) || (sDevice->data == NULL) || (fIndex >= ((VirtualDrive *)sDevice->data)->nFiles) || (mFile == NULL))
        return -1;
        
    MemoryFile *tmpFile = ((VirtualDrive *)sDevice->data)->mFile + fIndex;
    
    mFile->minute = tmpFile->minute;
    mFile->hour = tmpFile->hour;
    mFile->weekday = tmpFile->weekday;
    mFile->day = tmpFile->day;
    mFile->month = tmpFile->month;
    mFile->year = tmpFile->year;
    
    return 0;
}

int loadFile_Virtual(struct STORAGE_DEVICE *sDevice, Uint16 fIndex, MemoryFile *mFile)
{
    Uint32 size;
    void *data;
    VirtualDrive *vDrive;
    // check args
    if((mFile == NULL) || (sDevice == NULL) || (sDevice->data == NULL) || (fIndex >= ((VirtualDrive *)sDevice->data)->nFiles))
        return -1;
    vDrive = (VirtualDrive *)sDevice->data;
    // copy MemoryFile structure
    memcpy(mFile, vDrive->mFile + fIndex, sizeof(MemoryFile));
    // handle empty files
    size = getSize_Virtual(sDevice, fIndex);
    if(size > 0) {
        // allocate memory
        data = (void *)malloc(size);
        if(data == NULL) {
            mFile->size = 0;
            return -2;
        }    
        // load/copy data
        memcpy(data, (vDrive->mFile + fIndex)->data, size);
        mFile->data = data;
    }    
    return 0;
}

// append
int saveFile_Virtual(struct STORAGE_DEVICE *sDevice, MemoryFile *mFile)
{
    void *data;
    VirtualDrive *vDrive;
    // check args
    if((mFile == NULL) || (sDevice == NULL) || (sDevice->data == NULL))
        return -1;
    vDrive = (VirtualDrive *)sDevice->data;
    // allocate memory
        // a new virtual file
    if(vDrive->nFiles == 0)
        data = (MemoryFile *)malloc(sizeof(MemoryFile));
    else data = (MemoryFile *)realloc(vDrive->mFile, (vDrive->nFiles + 1) * sizeof(MemoryFile));
    if(data == NULL) 
        return -2;
    
    vDrive->mFile = data;
        // write MemoryFile structure
    memcpy(vDrive->mFile + vDrive->nFiles, mFile, sizeof(MemoryFile));
        // file content
    if(mFile->size > 0) {
        data = (void *)malloc(mFile->size);
        if(data == NULL) {
            // must TRY to free that MemoryFile
            if(vDrive->nFiles == 0)
                free(vDrive->mFile);
            else {
                data = (MemoryFile *)realloc(vDrive->mFile, (vDrive->nFiles) * sizeof(MemoryFile));
                if(data != NULL)
                    vDrive->mFile = data;
            }    
            return -2;
        }    
        // write data
        memcpy(data, mFile->data, mFile->size);
        (vDrive->mFile + vDrive->nFiles)->data = data;
    }  
    vDrive->nFiles++;

    return 0;
}

int deleteFile_Virtual(struct STORAGE_DEVICE *sDevice, Uint16 fIndex)
{
    int i;
    VirtualDrive *vDrive;
    MemoryFile *data;
    // check args
    if((sDevice == NULL) || (sDevice->data == NULL) || (fIndex >= ((VirtualDrive *)sDevice->data)->nFiles))
        return -1;
    vDrive = (VirtualDrive *)sDevice->data;
    // free file data
    if((vDrive->mFile + fIndex)->size > 0) {
        free((vDrive->mFile + fIndex)->data);
    }    
    // shift entries
    for(i = fIndex+1; i < vDrive->nFiles; i++) 
        memcpy(vDrive->mFile + i-1, vDrive->mFile + i, sizeof(MemoryFile));
        // free if it's empty now
    if(vDrive->nFiles == 1)
        free(vDrive->mFile);
    else {
        data = (MemoryFile *)realloc(vDrive->mFile, (vDrive->nFiles - 1) * sizeof(MemoryFile));
        if(data == NULL) {
            // last file is doubled,
            // so I MUST make it empty
            (vDrive->mFile + vDrive->nFiles - 1)->size = 0;
            return -2;
        }    
        vDrive->mFile = data;
    }    
    vDrive->nFiles--;

    return 0;
}

int format_Virtual(struct STORAGE_DEVICE *sDevice)
{
    VirtualDrive *vDrive;
    MemoryFile *data;
    int i;
    // check args
    if((sDevice == NULL) || (sDevice->data == NULL))
        return -1;
    vDrive = (VirtualDrive *)sDevice->data;
    // format/free
        // all file data
    for(i = 0; i < vDrive->nFiles; i++) {
//        were_here("fmt nF %i, sz %i, da %x", vDrive->nFiles, (vDrive->mFile + i)->size, (vDrive->mFile + i)->data);
        if((vDrive->mFile + i)->size > 0)
            free((vDrive->mFile + i)->data);
    }
//    were_here("fmt2");
        // all file MemoryFile
    if(vDrive->nFiles > 0)
        free(vDrive->mFile);

//    vDrive->mFile = NULL;
    vDrive->nFiles = 0;
    sDevice->fIndex = 0;
//    were_here("fmt3");

	return 0;
}

