//                                                                                
//    Copyright (c) 2004/2005 Thomas Fuchs / The Rockin'-B, www.rockin-b.de       
//                                                                                
//    Permission is granted to anyone to use this software for any purpose        
//    and to redistribute it freely, subject to the following restrictions:       
//                                                                                
//    1.  The origin of this software must not be misrepresented. You must not    
//        claim that you wrote the original software. If you use this software    
//        in a product, an acknowledgment in the product documentation would be   
//        appreciated but is not required.                                        
//                                                                                
//    2.  ANY COMMERCIAL USE IS PROHIBITED. This includes that you must not use   
//        this software in the production of a commercial product. Only an explicit
//        permission by the author can allow you to do that.                      
//                                                                                
//    3.  Any modification applied to the software must be marked as such clearly.
//                                                                                
//    4.  This license may not be removed or altered from any distribution.       
//                                                                                
//                                                                                
//    			    NO WARRANTY                                                 
//                                                                                
//    THERE IS NO WARRANTY FOR THE PROGRAM.                                       
//    THE PROGRAM IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,               
//    EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,                 
//    THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE.                 
//    THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.
//    SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY    
//    SERVICING, REPAIR OR CORRECTION.                                            
//                                                                                
//    IN NO EVENT WILL THE COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES,         
//    INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
//    OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED   
//    TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY    
//    YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER  
//    PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE       
//    POSSIBILITY OF SUCH DAMAGES.                                                
//                                                                                

#include <SGL.H>

// adapted from MineSweeper

extern void clearScreen();
extern void waitRelease(Uint16 buttons);

#define dim_x   8
#define dim_y   8
static Sint8 chartbl[dim_y][dim_x] =    {
     {'A','B','C','D','E','F','G','H'},
     {'I','J','K','L','M','N','O','P'},
     {'Q','R','S','T','U','V','W','X'},
     {'Y','Z','a','b','c','d','e','f'},
     {'g','h','i','j','k','l','m','n'},
     {'o','p','q','r','s','t','u','v'},
     {'w','x','y','z','0','1','2','3'},
     {'4','5','6','7','8','9','_','.'},
};


void SGM_string_draw_cursor(Uint8 item_x, Uint8 item_y)
{
     Uint8 i, j;

     /* first delete the old cursor */
     for(i = 0; i < dim_y; i++)
          slPrint(" ", slLocate(16, 13 + i)); 
     for(j = 0; j < dim_x; j++)
          slPrint(" ", slLocate(18 + j, 11)); 

     /* then draw new one */
     slPrint("+", slLocate(16, 13 + item_y)); 
     slPrint("+", slLocate(18 + item_x, 11)); 
}

void SGM_string_draw_string(Uint8 *target, Uint8 length, Uint8 name_length)
{
    slPrint(target, slLocate(16, 7));
    slPrintFX(toFIXED(length - name_length), slLocate(22, 9));
    // draw string cursor
    slPrint("             ", slLocate(16, 8));
    slPrint("+", slLocate(16 + name_length, 8));

}

void SGM_string_draw(Uint8 item_x, Uint8 item_y, Uint8 length, Uint8 name_length, Uint8 *target, Uint8 *instructions)
{
    Uint8 i, j;
    Sint8 sign[2] = {' ', '\0'};

    clearScreen();

     slPrint("String Editor", slLocate(15,1));

    slPrint(instructions, slLocate(7,5));
    
    SGM_string_draw_string(target, length, name_length);
    slPrint("characters left:", slLocate(7, 9));
    
    /* print menu */
    for(i = 0; i < dim_y; i++) {
          for(j = 0; j < dim_x; j++) {
               sign[0] = chartbl[i][j];
               slPrint(sign, slLocate(18 + j,13 + i));
          }
    }
    SGM_string_draw_cursor(item_x, item_y);
     
    slPrint("Select character with D-Pad.", slLocate(7,23));
    slPrint("Select string position with L/R.", slLocate(7,24));
    slPrint("Press A,C to insert one character", slLocate(7,25));
    slPrint("and B to remove the last.", slLocate(7,26));
    slPrint("Finish filename with START.", slLocate(7,27));

    slPrint("   add: A/C   www.rockin-b.de   remove: B   ", slLocate(0, 29));

    return;
}



void SGM_string(Uint8 *instructions, Uint8 length, Uint8 *target) 
{
    Uint16 push;
    //     Uint8 name[length + 1];
    Uint8 name_length = length;
    Uint8 item_x = 0, item_y = 0;
    
    *(target + length) = '\0';
    
    /* search cursor position:
     * place it behind the existing name */
    while(((name_length - 1) >= 0) && (*(target + name_length - 1) == ' '))
      name_length--;
    
    //     name[length] = '\0';
    clearScreen();
    
    /* draw gui */
    SGM_string_draw(item_x, item_y, length, name_length, target, instructions);
    
    /* loop controller inputs */
    waitRelease(PER_DGT_ST | PER_DGT_TA | PER_DGT_TB | PER_DGT_TC);
    while(1) {
        getfirstPadInput(NULL, &push, NULL);
        
        if((push & PER_DGT_TL) == 0) {
            // string cursor left
            if(name_length > 0) {
              name_length--;
              SGM_string_draw_string(target, length, name_length);
            }                 
        } else if((push & PER_DGT_TR) == 0) {
            // string cursor right
            if(name_length < length) {
              name_length++;
              SGM_string_draw_string(target, length, name_length);
            }                 
        } else if((push & PER_DGT_KU) == 0) {
            /* item up */
            if(item_y == 0) item_y = 7;
            else item_y--;
            SGM_string_draw_cursor(item_x, item_y);
        } else if((push & PER_DGT_KD) == 0) {
            /* item down */
            if(item_y == 7) item_y = 0;
            else item_y++;
            SGM_string_draw_cursor(item_x, item_y);
        } else if((push & PER_DGT_KL) == 0) {
            /* item left */
            if(item_x == 0) item_x = 7;
            else item_x--;
            SGM_string_draw_cursor(item_x, item_y);
        } else if((push & PER_DGT_KR) == 0) {
            /* item right */
            if(item_x == 7) item_x = 0;
            else item_x++;
            SGM_string_draw_cursor(item_x, item_y);
        } else if((push & PER_DGT_TB) == 0) {
            /* delete one character */
            if(name_length > 0) {
              //name[--name_length] = ' ';
              *(target + --name_length) = '_'; // for filename, not needed for comment
              SGM_string_draw_string(target, length, name_length);
            }     
        } else if(((push & PER_DGT_TA) == 0) || ((push & PER_DGT_TC) == 0)) {
            /* select item */
            if(name_length < length) {
              //name[name_length++] = chartbl[item_y][item_x];
              *(target + name_length++) = chartbl[item_y][item_x];
              SGM_string_draw_string(target, length, name_length);
            }
        } else if((push & PER_DGT_ST) == 0) {
            /* wait for release */
            waitRelease(PER_DGT_ST | PER_DGT_TA | PER_DGT_TB | PER_DGT_TC | PER_DGT_TX | PER_DGT_TY | PER_DGT_TZ);
            return;
        }
        
        slSynch();
    }
}

