//                                                                                
//    Copyright (c) 2004/2005 Thomas Fuchs / The Rockin'-B, www.rockin-b.de       
//                                                                                
//    Permission is granted to anyone to use this software for any purpose        
//    and to redistribute it freely, subject to the following restrictions:       
//                                                                                
//    1.  The origin of this software must not be misrepresented. You must not    
//        claim that you wrote the original software. If you use this software    
//        in a product, an acknowledgment in the product documentation would be   
//        appreciated but is not required.                                        
//                                                                                
//    2.  ANY COMMERCIAL USE IS PROHIBITED. This includes that you must not use   
//        this software in the production of a commercial product. Only an explicit
//        permission by the author can allow you to do that.                      
//                                                                                
//    3.  Any modification applied to the software must be marked as such clearly.
//                                                                                
//    4.  This license may not be removed or altered from any distribution.       
//                                                                                
//                                                                                
//    			    NO WARRANTY                                                 
//                                                                                
//    THERE IS NO WARRANTY FOR THE PROGRAM.                                       
//    THE PROGRAM IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,               
//    EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,                 
//    THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE.                 
//    THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.
//    SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY    
//    SERVICING, REPAIR OR CORRECTION.                                            
//                                                                                
//    IN NO EVENT WILL THE COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES,         
//    INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
//    OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED   
//    TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY    
//    YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER  
//    PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE       
//    POSSIBILITY OF SUCH DAMAGES.                                                
//                                                                                


// Common error codes:
// -1   wrong args
// -2   memory allocation error
// -3   process error


#include <SGL.H>

#include <SGM_def.h>
#include <SGM_popup.h>
#include <SGM_BZip2.h>
#include <SGM_device.h>
#include <SGM_menu.h>



extern void SGM_help();

/****************************************************************************************************/
/* common functions *********************************************************************************/
/****************************************************************************************************/

// check if pad is connected
// and return first pad with input
// multi tap compatible, up to 12 pads
//
// return: 	-1 		- no pad connected
//			-2 		- no input
//			else	- pad index
Sint8 getFirstPadIndex()
{
    Sint8 index;

    // is there even a pad connected?
    // check number of attached peripherals
    if((Per_Connect1 + Per_Connect2) == 0)
        return -1;
        
        // check up to 6 pads on left connector
    for(index = 0; index < Per_Connect1; index++) 
        if((Smpc_Peripheral[index].data != 0xFFFF) || (Smpc_Peripheral[index].pull != 0xFFFF))
            return index;

        // check up to 6 pads on right connector
    for(index = 0; index < Per_Connect2; index++) 
        if((Smpc_Peripheral[15+index].data != 0xFFFF) || (Smpc_Peripheral[15+index].pull != 0xFFFF))
            return 15+index;
            
        // no inputs found!
        // that is no error
    return -2;
}

void checkExitHere(Uint16 data)
{
    // check exit
    if(((data & PER_DGT_ST) == 0) && ((data & PER_DGT_TA) == 0) && ((data & PER_DGT_TB) == 0) && ((data & PER_DGT_TC) == 0)) {
        // return to demo demo loader
        // or activate multiplayer
        SYS_Exit(0);
    }
}

// return index and write button data
Sint8 getfirstPadInput(Uint16 *data, Uint16 *push, Uint16 *pull)
{
    Sint8 index = getFirstPadIndex();
    // pad found?    
    if(index == -1) {
        // no pad connected
        clearScreen();
        slPrint("!!! NO CONTROLLER CONNECTED !!!", slLocate(6, 14));
        // wait for a pad being connected
        do {
            index = getFirstPadIndex();
            slSynch();
        } while(index == -1);
    }
    if(index >= 0) {
        checkExitHere(Smpc_Peripheral[index].data);
        if(data != NULL)
            *data = Smpc_Peripheral[index].data;
        if(push != NULL)
            *push = Smpc_Peripheral[index].push;
        if(pull != NULL)
            *pull = Smpc_Peripheral[index].pull;
    } else {
        if(data != NULL)
            *data = 0xFFFF;
        if(push != NULL)
            *push = 0xFFFF;
        if(pull != NULL)
            *pull = 0xFFFF;
    }
    
    return index;
}

//void checkExit()
//{
//    Uint16 data;
//    // checkexit called inside....
//    if(getfirstPadInput(&data, NULL, NULL) >= 0)
//        checkExitHere(data);
//}

void clearScreen()
{
    int i;
    for(i = 0; i < 30; i++)
        slPrint("                                            ", slLocate(0, i));
}

void waitRelease(Uint16 buttons)
{
    Uint16 data, pull;
    // wait release
    do {
        slSynch();
        getfirstPadInput(&data, NULL, &pull);
    } while( ((data & buttons) != buttons) && ((pull & buttons) != buttons));
//    } while(((data & PER_DGT_ST) == 0) || ((data & PER_DGT_TA) == 0) || ((data & PER_DGT_TB) == 0) || ((data & PER_DGT_TC) == 0));
}



int memoryEqual(Uint8 *a, Uint8 *b, Uint32 n)
{
    int o;
    for(o = 0; o < n; o++) {
        if(*(a+o) != *(b+o))
            return 0;
    }
    return 1;
}


// memory debug

#ifdef DEBUG_BGET
#   ifndef _
#       include <bget.h>
#   endif
void *dbg_bget(bufsize size)
{
    void *ptr = bget(size);
    waitRelease(PER_DGT_ST | PER_DGT_TA);
    were_here("bget: addr %x, size %i", ptr, size);
    return ptr;
}
void *dbg_bgetz(bufsize size)
{
    void *ptr = bgetz(size);
    waitRelease(PER_DGT_ST | PER_DGT_TA);
    were_here("bgetz: addr %x, size %i", ptr, size);
    return ptr;
}
void *dbg_bgetr(void *buffer, bufsize newsize)
{
    void *ptr = bgetr(buffer, newsize);
    waitRelease(PER_DGT_ST | PER_DGT_TA);
    were_here("bgetr: addr2 %x,size2 %i", ptr, newsize);
    return ptr;
}
void dbg_brel(void *buf)
{
    brel(buf);
    waitRelease(PER_DGT_ST | PER_DGT_TA);
    were_here("brel: addr %x", buf);
}
#endif // DEBUG_BGET

/****************************************************************************************************/
/* flash Action Replay from ExCyber *****************************************************************/
/****************************************************************************************************/

Uint32 len = 0;
#define ss_main         flash_main
#define ARROM_LEN       len
#define arrom_image     NULL
#include "arflash/main.c"
#undef ss_main

// this exits the app in either case
int flash(char *filename) {
    int i;
    // check args
    if(filename == NULL)
        return -1;
    // load the flash rom
    arrom = (Uint16 *)LOW_MEM_START;
   	len = GFS_Load(GFS_NameToId(filename), 0, (Uint32 *)arrom, LOW_MEM_SIZE);
    if(len <= 0) {
        SGM_confirm(0xFFFF, CONFIRM_LOAD_ERROR, filename);
        return -3;
    }
    clearScreen();
    // flash parameters
    // run flash app
    flash_main();
    // never reach
    return 0;
}

/****************************************************************************************************/
/* drawing main screen ******************************************************************************/
/****************************************************************************************************/

#define LINES_PER_SLOT  9
void SGM_drawFiles(StorageDevice *sDevice, Uint8 y)
{
    Uint16 fIndex = (sDevice->fIndex / LINES_PER_SLOT ) * LINES_PER_SLOT;
    Uint8 i;
    MemoryFile tmpFile;
    char date[] = "05/08/31";
    
    for(i = 0; (i < LINES_PER_SLOT) && (fIndex < sDevice->getFileNum(sDevice)); i++, fIndex++) {
        slPrint(sDevice->getName(sDevice, fIndex), slLocate(1, y + i));
        slPrint(sDevice->getComment(sDevice, fIndex), slLocate(1 + 12 + 1, y + i));
        // swap date & size to avoid overwriting
        slPrintFX(((sDevice->getSize(sDevice, fIndex) + 32) / 64) << 16, slLocate(1 + 12 + 1 + 11 + 1 + 8 + 1, y + i));
        sDevice->getDate(sDevice, fIndex, &tmpFile);
        date[0] = '0' + (tmpFile.year % 100) / 10;
        date[1] = '0' + tmpFile.year % 10;
        date[3] = '0' + tmpFile.month / 10;
        date[4] = '0' + tmpFile.month % 10;
        date[6] = '0' + tmpFile.day / 10;
        date[7] = '0' + tmpFile.day % 10;
        slPrint(date, slLocate(1 + 12 + 1 + 11 + 0, y + i));
    }
}

void SGM_drawCursor(Uint8 selectedFile, Uint8 y)
{
    Uint8 i;

    slPrint("|------------------------------------------|", slLocate(0, y));
    for(i = 0; i < LINES_PER_SLOT; i++) {
        // file selected?
        if(selectedFile == i) {
            slPrint(">", slLocate(0, y + 1 + i));
            slPrint("-", slLocate(1 + 12 + 1 - 1, y + 1 + i));
            slPrint("-", slLocate(1 + 12 + 1 + 11 + 1 - 2, y + 1 + i));
            slPrint("<", slLocate(43, y + 1 + i));
        } else {
            slPrint("|", slLocate(0, y + 1 + i));
            slPrint("|", slLocate(43, y + 1 + i));
        }
    }
    slPrint("|------------------------------------------|", slLocate(0, y + LINES_PER_SLOT + 1));
}

void SGM_drawDeviceSpecs(Uint8 slot, Uint8 y)
{
    slPrintFX(activeDevices[slot] << 16, slLocate(0,y));
    slPrint((device + activeDevices[slot])->name, slLocate(7, y));
    slPrintFX((( (device + activeDevices[slot])->getFreeBytes(device + activeDevices[slot]) + 32) / 64) << 16, slLocate(1 + 12 + 1 + 11 + 1 + 8 + 1, y));
    slPrint("free blocks:", slLocate(1 + 12 + 1 + 11 - 1, y));
}

void SGM_draw()
{
    Uint8 i;
    
    clearScreen();
    // redraw
    slPrint("Rockin'-B's Save Game Manager", slLocate(5, 1));
    SGM_drawFiles(device + activeDevices[0], 15-1-LINES_PER_SLOT);
    slPrint("Filename     Comment    Date      Size    ", slLocate(1, 15));
    SGM_drawFiles(device + activeDevices[1], 17);
    // cursor
    if(activeSlot == 0)
        SGM_drawCursor((device + activeDevices[0])->fIndex % LINES_PER_SLOT, 15-1-LINES_PER_SLOT-1);
    else SGM_drawCursor((device + activeDevices[1])->fIndex % LINES_PER_SLOT, 15+1);
    // device stats
    SGM_drawDeviceSpecs(0, 15-1-LINES_PER_SLOT-1-1);
    SGM_drawDeviceSpecs(1, 15+1+LINES_PER_SLOT+1+1);
    slPrint("www.rockin-b.de", slLocate(14, 29));
#ifdef USE_BGET
    {
        bufsize curalloc, totfree, maxfree;
        long nget, nrel;
        
	    bstats(&curalloc, &totfree, &maxfree, &nget, &nrel);
	    slPrintHex(curalloc, slLocate(5, 29));
	    slPrintHex(totfree, slLocate(30, 29));
    }  
#endif              
}


/****************************************************************************************************/
/* Save Game Manager ********************************************************************************/
/****************************************************************************************************/

Uint8 SGM_inputs(Uint16 push, Uint16 data)
{
    // only one command at a time
    // and priority to saver commands
    // for savety of usage
    if((push & PER_DGT_TA) == 0) {
    // 1. copy
        SGM_copy();
    } else if((push & PER_DGT_TY) == 0) {
    // 2. open virtual drive
        SGM_open();
    } else if((push & PER_DGT_TZ) == 0) {
    // 3. save device as compressed virtual drive file
        SGM_save();
    } else if((push & PER_DGT_TB) == 0) {
    // 4. delete file
        SGM_delete();
    } else if((push & PER_DGT_TC) == 0) {
    // 5. format
        SGM_format();
    } else if((push & PER_DGT_TX) == 0) {
    // 6. buffer swap
        if(activeSlot == 0) {
            activeSlot = 1;
            inactiveSlot = 0;
        } else {
            activeSlot = 0;
            inactiveSlot = 1;
        }
    } else if((push & PER_DGT_KL) == 0) {
    // 7. device change
        Uint8 tmp = activeDevices[activeSlot];
        // previous
        if(activeDevices[activeSlot]-- == 0)
            activeDevices[activeSlot] = nDevices-1;
        // correct equal devices
        if(activeDevices[activeSlot] == activeDevices[inactiveSlot]) {
            // swap devices
            activeDevices[inactiveSlot] = tmp;
        }
    } else if((push & PER_DGT_KR) == 0) {
        // next
        Uint8 tmp = activeDevices[activeSlot];
        if(++activeDevices[activeSlot] >= nDevices)
            activeDevices[activeSlot] = 0;
        // correct equal devices
        if(activeDevices[activeSlot] == activeDevices[inactiveSlot]) {
            // swap devices
            activeDevices[inactiveSlot] = tmp;
        }
    } else if(((data & PER_DGT_TL) == 0) || ((push & PER_DGT_KU) == 0)) {
    // 8. file select change
        // down
        if(aDevice->fIndex > 0)
            aDevice->fIndex--;
    } else if(((data & PER_DGT_TR) == 0) || ((push & PER_DGT_KD) == 0)) {
        // up
        if((aDevice->fIndex + 1) < aDevice->getFileNum(aDevice))
            aDevice->fIndex++;
    } else return 0;
    
    return 1;
}

void SGM_loop()
{
    Uint16 push, data;
    
    SGM_help();
    SGM_draw();
    while(1) {
    // this does not work,
    // just as it didn't in SoundPlayer
//        // track disk change
//        if((CDC_GetHirqReq() & CDC_HIRQ_DCHG) != 0) {
//            updateDir_CD(device + DEVICE_CD_DRIVE);
//            CDC_ClrHirqReq(~CDC_HIRQ_DCHG);
//        }
        // get inputs
        getfirstPadInput(&data, &push, NULL);
        // check help display
        if((push & PER_DGT_ST) == 0) {
            SGM_menu();
//            SGM_help();
            SGM_draw();
        } else {
            // redraw only if inputs occured
            if(SGM_inputs(push, data)) {
                checkVirtualDevices();
                SGM_draw();
            }
            slSynch();
        }
    }
}

/****************************************************************************************************/
/* background stuff *********************************************************************************/
/****************************************************************************************************/

// line-by-line transfer to vram
void img2VRAM(Uint16 *vram, Uint16 vram_width, Uint16 *img, Uint16 img_height, Uint16 img_width)
{
    int i;
    Uint32 oVRAM = 0;
    Uint32 oIMG = 0;
    
    for(i = 0; i < img_height; i++) {
        memcpy((void *)(vram + oVRAM), (void *)(img + oIMG), img_width*sizeof(Uint16));
        oVRAM += vram_width;
        oIMG += img_width;
    }    
} 

void init_VDP2()
{
// the address for background color in VRAM is taken from various examples,
// don't know why
// !!! beware, 1st try the bgcol was the only thing to see on real Saturn
// what exactly is the back_col_adr? maybe the color at this adress is 
// interpreted as background color and wheereever such color occurs,
// it's replaced by the given one
// !! all i see is the background color....
    slBack1ColSet((void *)BACK_CRAM, CD_Black);
    
    
// the debug text lib take the NBG0 scroll <- the only one with 24bits color depth
// priority = 0 -> scroll not displayed
// take farest away    
    slPriorityNbg0(7);
    slPriorityNbg1(6);
//    slPrioritySpr0-7(0);    

// we want bitmap scrolls
// only NBG0 and NBG1 can handle bitmap mode
// color modes:
//      palettet:           COL_TYPE_16, COL_TYPE_256, COL_TYPE_2048
//      rgb:                COL_TYPE_32768, COL_TYPE_1M
// bitmap sizes(BM_HxV):    BM_512x256, BM_512x512, BM_1024x256, BM_1024x512
    slBitMapNbg1(COL_TYPE_32768, BM_512x256, (void *)VDP2_VRAM_A0);
//    memset((void *)VDP2_VRAM_A0, CD_Black, 256*512*2);

// monitor position in scroll coordinate space
// sets the position of monitors topleft corner(fixed format)
//    slScrPosNbg1(toFIXED(-33.680/2.0), toFIXED(0.0));
    slScrPosNbg1(toFIXED(0.0), toFIXED(0.0));

// reduction setting: only NGB0 and NBG1 are able to RBG, too)
// modes:   ZOOM_1, ZOOM_HALF, ZOOM_QUATER      each untill x256
    slZoomModeNbg1(ZOOM_1);
    
// zoom
// fixed point reziprocal of scale ratio in x and y direction
    slZoomNbg1(toFIXED(1.0), toFIXED(1.0));            

// registeres scrolls
// call only after all scroll function settings
// returns 0 or -1(ERROR)
    slScrAutoDisp(NBG0ON | NBG1ON);
}

void ss_main(void)
{
    slInitSystem(TV_352x240, NULL, 2);
    
    // VDP 2
    slTVOff();

    init_VDP2();
    
    slTVOn();

    init_CD();

//    were_here("before image");
    // load from CD and display the
    // background image
  	if(GFS_Load(GFS_NameToId("LAVA.BIN"), 0, (Uint32 *)LOW_MEM_START, LOW_MEM_SIZE) > 0)
        img2VRAM((Uint16 *)VDP2_VRAM_A0, 512, (Uint16 *)(LOW_MEM_START), 240, 352);
//    were_here("after image");
    
    // init memory allocation lib: BGET
#ifdef USE_BGET
        // VRAM is last portion (SGL/DOCS/MEMORY.TXT)
        // not cache-through
        // 466.232 Bytes
    bpool(0x5C0E1C0, 0x5C7FEF8-0x5C0E1C0);
        // cartridge RAM, be quiet
    CartID = RB_CartRAM_init(0, 1);
    if(CartID == CARTRAM_ID_32M) {
        // one continuous bank of cart RAM
        bpool(CartRAM0, CartRAMsize << 1);
    } else if(CartID == CARTRAM_ID_8M) {
        // two seperate, non-continuious cart RAM banks
        bpool(CartRAM0, CartRAMsize);
        bpool(CartRAM1, CartRAMsize);
    }
        // low work RAM
    bpool(LOW_MEM_START, LOW_MEM_SIZE);
        // free portion of high work RAM up to SGL variable space (SGL/DOCS/MEMORY.TXT)
    bpool((Uint32)(&_bend), 0x060E0000 - ((Uint32)(&_bend)));   
    
#endif    

    init_BackUp();
    init_Devices();
    
    SGM_loop();

    // return to demo demo loader
    // or activate multiplayer
    SYS_Exit(0);
}


// This is the content of cinit.c
// It eases your business when compiling C++ programs
// if you have it here.
#if 0
#include <SGL.H>
/*********************************/
/* taken from cinit.c */
/*********************************/
#define		SystemWork		0x060ffc00		/* System Variable Address */
#define		SystemSize		(0x06100000-0x060ffc00)		/* System Variable Size */
/* sl.lnk Ŏw肵.bssZNV̊JnAIV{ */
extern Uint32 _bstart, _bend;
 
int main( void )
{
	Uint8	*dst;
	Uint32	i; 

	/* 1.Zero Set .bss Section */
	for( dst = (Uint8 *)&_bstart; dst < (Uint8 *)&_bend; dst++ ) {
		*dst = 0;
	}
	/* 2.ROM has data at end of text; copy it. */

	/* 3.SGL System Variable Clear */
	for( dst = (Uint8 *)SystemWork, i = 0;i < SystemSize; i++) {
		*dst = 0;
	}

	/* Application Call */
	ss_main();
	return 1;
}
#endif
